@@ -1,8 +1,8 @@
 /*****************************************************************************
  * podcast.c : podcast playlist imports
  *****************************************************************************
- * Copyright (C) 2005 the VideoLAN team
- * $Id: 922c0a27251cfbf363c59aab13516d81374e7872 $
+ * Copyright (C) 2005-2009 the VideoLAN team
+ * $Id: df6736e1ea498417079bb562dded31363611c4ca $
  *
  * Authors: Antoine Cellerier <dionoea -at- videolan -dot- org>
  *
@@ -32,20 +32,14 @@
 #include <vlc_demux.h>
 
 #include "playlist.h"
-#include "vlc_xml.h"
-
-struct demux_sys_t
-{
-    char *psz_prefix;
-    xml_t *p_xml;
-    xml_reader_t *p_xml_reader;
-};
+#include <vlc_xml.h>
 
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
 static int Demux( demux_t *p_demux);
 static int Control( demux_t *p_demux, int i_query, va_list args );
+static mtime_t strTimeToMTime( const char *psz );
 
 /*****************************************************************************
  * Import_podcast: main import function
@@ -57,10 +51,9 @@ int Import_podcast( vlc_object_t *p_this )
     if( !demux_IsForced( p_demux, "podcast" ) )
         return VLC_EGENERIC;
 
-    STANDARD_DEMUX_INIT_MSG( "using podcast reader" );
-    p_demux->p_sys->psz_prefix = FindPrefix( p_demux );
-    p_demux->p_sys->p_xml = NULL;
-    p_demux->p_sys->p_xml_reader = NULL;
+    p_demux->pf_demux = Demux;
+    p_demux->pf_control = Control;
+    msg_Dbg( p_demux, "using podcast reader" );
 
     return VLC_SUCCESS;
 }
@@ -70,26 +63,18 @@ int Import_podcast( vlc_object_t *p_this )
  *****************************************************************************/
 void Close_podcast( vlc_object_t *p_this )
 {
-    demux_t *p_demux = (demux_t *)p_this;
-    demux_sys_t *p_sys = p_demux->p_sys;
-
-    free( p_sys->psz_prefix );
-    if( p_sys->p_xml_reader ) xml_ReaderDelete( p_sys->p_xml, p_sys->p_xml_reader );
-    if( p_sys->p_xml ) xml_Delete( p_sys->p_xml );
-    free( p_sys );
+    (void)p_this;
 }
 
 /* "specs" : http://phobos.apple.com/static/iTunesRSS.html */
 static int Demux( demux_t *p_demux )
 {
-    demux_sys_t *p_sys = p_demux->p_sys;
-
     bool b_item = false;
     bool b_image = false;
     int i_ret;
 
     xml_t *p_xml;
-    xml_reader_t *p_xml_reader;
+    xml_reader_t *p_xml_reader = NULL;
     char *psz_elname = NULL;
     char *psz_item_mrl = NULL;
     char *psz_item_size = NULL;
@@ -102,36 +87,37 @@ static int Demux( demux_t *p_demux )
     char *psz_item_keywords = NULL;
     char *psz_item_subtitle = NULL;
     char *psz_item_summary = NULL;
+    char *psz_art_url = NULL;
     int i_type;
     input_item_t *p_input;
+    input_item_node_t *p_subitems = NULL;
 
-    INIT_PLAYLIST_STUFF;
+    input_item_t *p_current_input = GetCurrentItem(p_demux);
 
-    p_xml = p_sys->p_xml = xml_Create( p_demux );
-    if( !p_xml ) return -1;
-
-/*    psz_elname = stream_ReadLine( p_demux->s );
-    if( psz_elname ) free( psz_elname );
-    psz_elname = 0;*/
+    p_xml = xml_Create( p_demux );
+    if( !p_xml )
+        goto error;
 
     p_xml_reader = xml_ReaderCreate( p_xml, p_demux->s );
-    if( !p_xml_reader ) return -1;
-    p_sys->p_xml_reader = p_xml_reader;
+    if( !p_xml_reader )
+        goto error;
 
     /* xml */
     /* check root node */
     if( xml_ReaderRead( p_xml_reader ) != 1 )
     {
         msg_Err( p_demux, "invalid file (no root node)" );
-        return -1;
+        goto error;
     }
 
     while( xml_ReaderNodeType( p_xml_reader ) == XML_READER_NONE )
+    {
         if( xml_ReaderRead( p_xml_reader ) != 1 )
         {
             msg_Err( p_demux, "invalid file (no root node)" );
-            return -1;
+            goto error;
         }
+    }
 
     if( xml_ReaderNodeType( p_xml_reader ) != XML_READER_STARTELEM ||
         ( psz_elname = xml_ReaderName( p_xml_reader ) ) == NULL ||
@@ -139,10 +125,11 @@ static int Demux( demux_t *p_demux )
     {
         msg_Err( p_demux, "invalid root node %i, %s",
                  xml_ReaderNodeType( p_xml_reader ), psz_elname );
-        free( psz_elname );
-        return -1;
+        goto error;
     }
-    free( psz_elname ); psz_elname = NULL;
+    FREENULL( psz_elname );
+
+    p_subitems = input_item_node_Create( p_current_input );
 
     while( (i_ret = xml_ReaderRead( p_xml_reader )) == 1 )
     {
@@ -152,15 +139,15 @@ static int Demux( demux_t *p_demux )
         {
             // Error
             case -1:
-                return -1;
-                break;
+                goto error;
 
             case XML_READER_STARTELEM:
             {
                 // Read the element name
                 free( psz_elname );
                 psz_elname = xml_ReaderName( p_xml_reader );
-                if( !psz_elname ) return -1;
+                if( !psz_elname )
+                    goto error;
 
                 if( !strcmp( psz_elname, "item" ) )
                 {
@@ -168,7 +155,7 @@ static int Demux( demux_t *p_demux )
                 }
                 else if( !strcmp( psz_elname, "image" ) )
                 {
-                    b_item = true;
+                    b_image = true;
                 }
 
                 // Read the attributes
@@ -180,100 +167,123 @@ static int Demux( demux_t *p_demux )
                     {
                         free( psz_name );
                         free( psz_value );
-                        free( psz_elname );
-                        return -1;
+                        goto error;
                     }
-                    if( !strcmp( psz_elname, "enclosure" ) &&
-                        !strcmp( psz_name, "url" ) )
+
+                    if( !strcmp( psz_elname, "enclosure" ) )
                     {
-                        free( psz_item_mrl );
-                        psz_item_mrl = strdup( psz_value );
-                    }
-                    else if( !strcmp( psz_elname, "enclosure" ) &&
-                        !strcmp( psz_name, "length" ) )
-                    {
-                        free( psz_item_size );
-                        psz_item_size = strdup( psz_value );
-                    }
-                    else if( !strcmp( psz_elname, "enclosure" ) &&
-                        !strcmp( psz_name, "type" ) )
-                    {
-                        free( psz_item_type );
-                        psz_item_type = strdup( psz_value );
+                        if( !strcmp( psz_name, "url" ) )
+                        {
+                            free( psz_item_mrl );
+                            psz_item_mrl = psz_value;
+                        }
+                        else if( !strcmp( psz_name, "length" ) )
+                        {
+                            free( psz_item_size );
+                            psz_item_size = psz_value;
+                        }
+                        else if( !strcmp( psz_name, "type" ) )
+                        {
+                            free( psz_item_type );
+                            psz_item_type = psz_value;
+                        }
+                        else
+                        {
+                            msg_Dbg( p_demux,"unhandled attribute %s in element %s",
+                                     psz_name, psz_elname );
+                            free( psz_value );
+                        }
                     }
                     else
                     {
-                        msg_Dbg( p_demux,"unhandled attribure %s in element %s",
+                        msg_Dbg( p_demux,"unhandled attribute %s in element %s",
                                   psz_name, psz_elname );
+                        free( psz_value );
                     }
                     free( psz_name );
-                    free( psz_value );
                 }
                 break;
             }
             case XML_READER_TEXT:
             {
-#define SET_DATA( field, name ) else if( b_item == true \
-                && !strcmp( psz_elname, name ) ) \
-                { \
-                    field = strdup( psz_text ); \
-                }
+                if(!psz_elname) break;
+
                 char *psz_text = xml_ReaderValue( p_xml_reader );
+
+#define SET_DATA( field, name )                 \
+    else if( !strcmp( psz_elname, name ) )      \
+    {                                           \
+        field = psz_text;                       \
+    }
                 /* item specific meta data */
-                if( b_item == true && !strcmp( psz_elname, "title" ) )
+                if( b_item == true )
                 {
-                    psz_item_name = strdup( psz_text );
+                    if( !strcmp( psz_elname, "title" ) )
+                    {
+                        psz_item_name = psz_text;
+                    }
+                    else if( !strcmp( psz_elname, "itunes:author" ) ||
+                             !strcmp( psz_elname, "author" ) )
+                    { /* <author> isn't standard iTunes podcast stuff */
+                        psz_item_author = psz_text;
+                    }
+                    else if( !strcmp( psz_elname, "itunes:summary" ) ||
+                             !strcmp( psz_elname, "description" ) )
+                    { /* <description> isn't standard iTunes podcast stuff */
+                        psz_item_summary = psz_text;
+                    }
+                    SET_DATA( psz_item_date, "pubDate" )
+                    SET_DATA( psz_item_category, "itunes:category" )
+                    SET_DATA( psz_item_duration, "itunes:duration" )
+                    SET_DATA( psz_item_keywords, "itunes:keywords" )
+                    SET_DATA( psz_item_subtitle, "itunes:subtitle" )
+                    else
+                        free( psz_text );
                 }
-                else if( b_item == true
-                         && ( !strcmp( psz_elname, "itunes:author" )
-                            ||!strcmp( psz_elname, "author" ) ) )
-                { /* <author> isn't standard iTunes podcast stuff */
-                    psz_item_author = strdup( psz_text );
-                }
-                else if( b_item == true
-                         && ( !strcmp( psz_elname, "itunes:summary" )
-                            ||!strcmp( psz_elname, "description" ) ) )
-                { /* <description> isn't standard iTunes podcast stuff */
-                    psz_item_summary = strdup( psz_text );
-                }
-                SET_DATA( psz_item_date, "pubDate" )
-                SET_DATA( psz_item_category, "itunes:category" )
-                SET_DATA( psz_item_duration, "itunes:duration" )
-                SET_DATA( psz_item_keywords, "itunes:keywords" )
-                SET_DATA( psz_item_subtitle, "itunes:subtitle" )
 #undef SET_DATA
+
                 /* toplevel meta data */
-                else if( b_item == false && b_image == false
-                         && !strcmp( psz_elname, "title" ) )
+                else if( b_image == false )
                 {
-                    input_item_SetName( p_current_input, psz_text );
-                }
+                    if( !strcmp( psz_elname, "title" ) )
+                    {
+                        input_item_SetName( p_current_input, psz_text );
+                    }
 #define ADD_GINFO( info, name ) \
-    else if( !b_item && !b_image && !strcmp( psz_elname, name ) ) \
+    else if( !strcmp( psz_elname, name ) ) \
     { \
         input_item_AddInfo( p_current_input, _("Podcast Info"), \
                                 _( info ), "%s", psz_text ); \
     }
-                ADD_GINFO( "Podcast Link", "link" )
-                ADD_GINFO( "Podcast Copyright", "copyright" )
-                ADD_GINFO( "Podcast Category", "itunes:category" )
-                ADD_GINFO( "Podcast Keywords", "itunes:keywords" )
-                ADD_GINFO( "Podcast Subtitle", "itunes:subtitle" )
+                    ADD_GINFO( "Podcast Link", "link" )
+                    ADD_GINFO( "Podcast Copyright", "copyright" )
+                    ADD_GINFO( "Podcast Category", "itunes:category" )
+                    ADD_GINFO( "Podcast Keywords", "itunes:keywords" )
+                    ADD_GINFO( "Podcast Subtitle", "itunes:subtitle" )
 #undef ADD_GINFO
-                else if( b_item == false && b_image == false
-                         && ( !strcmp( psz_elname, "itunes:summary" )
-                            ||!strcmp( psz_elname, "description" ) ) )
-                { /* <description> isn't standard iTunes podcast stuff */
-                    input_item_AddInfo( p_current_input,
-                             _( "Podcast Info" ), _( "Podcast Summary" ),
-                             "%s", psz_text );
+                    else if( !strcmp( psz_elname, "itunes:summary" ) ||
+                             !strcmp( psz_elname, "description" ) )
+                    { /* <description> isn't standard iTunes podcast stuff */
+                        input_item_AddInfo( p_current_input,
+                            _( "Podcast Info" ), _( "Podcast Summary" ),
+                            "%s", psz_text );
+                    }
+                    free( psz_text );
                 }
                 else
                 {
-                    msg_Dbg( p_demux, "unhandled text in element '%s'",
-                              psz_elname );
+                    if( !strcmp( psz_elname, "url" ) )
+                    {
+                        free( psz_art_url );
+                        psz_art_url = psz_text;
+                    }
+                    else
+                    {
+                        msg_Dbg( p_demux, "unhandled text in element '%s'",
+                                 psz_elname );
+                        free( psz_text );
+                    }
                 }
-                free( psz_text );
                 break;
             }
             // End element
@@ -282,14 +292,14 @@ static int Demux( demux_t *p_demux )
                 // Read the element name
                 free( psz_elname );
                 psz_elname = xml_ReaderName( p_xml_reader );
-                if( !psz_elname ) return -1;
+                if( !psz_elname )
+                    goto error;
                 if( !strcmp( psz_elname, "item" ) )
                 {
                     if( psz_item_mrl == NULL )
                     {
                         msg_Err( p_demux, "invalid XML (no enclosure markup)" );
-                        free( psz_elname );
-                        return -1;
+                        goto error;
                     }
                     p_input = input_item_New( p_demux, psz_item_mrl, psz_item_name );
                     if( p_input == NULL ) break;
@@ -305,6 +315,14 @@ static int Demux( demux_t *p_demux )
                     ADD_INFO( "Podcast Summary", psz_item_summary );
                     ADD_INFO( "Podcast Type", psz_item_type );
 #undef ADD_INFO
+
+                    /* Set the duration if available */
+                    if( psz_item_duration )
+                        input_item_SetDuration( p_input, strTimeToMTime( psz_item_duration ) );
+                    /* Add the global art url to this item, if any */
+                    if( psz_art_url )
+                        input_item_SetArtURL( p_input, psz_art_url );
+
                     if( psz_item_size )
                     {
                         input_item_AddInfo( p_input,
@@ -313,7 +331,7 @@ static int Demux( demux_t *p_demux )
                                                 "%s bytes",
                                                 psz_item_size );
                     }
-                    input_item_AddSubItem( p_current_input, p_input );
+                    input_item_node_AppendItem( p_subitems, p_input );
                     vlc_gc_decref( p_input );
                     FREENULL( psz_item_name );
                     FREENULL( psz_item_mrl );
@@ -345,10 +363,39 @@ static int Demux( demux_t *p_demux )
         msg_Warn( p_demux, "error while parsing data" );
     }
 
+    free( psz_art_url );
+    free( psz_elname );
+    xml_ReaderDelete( p_xml, p_xml_reader );
+    xml_Delete( p_xml );
+
+    input_item_node_PostAndDelete( p_subitems );
+    vlc_gc_decref(p_current_input);
+    return 0; /* Needed for correct operation of go back */
+
+error:
+    free( psz_item_name );
+    free( psz_item_mrl );
+    free( psz_item_size );
+    free( psz_item_type );
+    free( psz_item_date );
+    free( psz_item_author );
+    free( psz_item_category );
+    free( psz_item_duration );
+    free( psz_item_keywords );
+    free( psz_item_subtitle );
+    free( psz_item_summary );
+    free( psz_art_url );
     free( psz_elname );
 
-    HANDLE_PLAY_AND_RELEASE;
-    return 0; /* Needed for correct operation of go back */
+    if( p_xml_reader )
+        xml_ReaderDelete( p_xml, p_xml_reader );
+    if( p_xml )
+        xml_Delete( p_xml );
+    if( p_subitems )
+        input_item_node_Delete( p_subitems );
+
+    vlc_gc_decref(p_current_input);
+    return -1;
 }
 
 static int Control( demux_t *p_demux, int i_query, va_list args )
@@ -356,3 +403,18 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
     VLC_UNUSED(p_demux); VLC_UNUSED(i_query); VLC_UNUSED(args);
     return VLC_EGENERIC;
 }
+
+static mtime_t strTimeToMTime( const char *psz )
+{
+    int h, m, s;
+    switch( sscanf( psz, "%u:%u:%u", &h, &m, &s ) )
+    {
+    case 3:
+        return (mtime_t)( ( h*60 + m )*60 + s ) * 1000000;
+    case 2:
+        return (mtime_t)( h*60 + m ) * 1000000;
+        break;
+    default:
+        return -1;
+    }
+}
