@@ -2,7 +2,7 @@
  * intf.m: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2002-2009 the VideoLAN team
- * $Id: b4eab05580144e79fbb16e7b711343cb7ba92c78 $
+ * $Id: 6a06c054e901443ce4c0cc8efd2f7217b5629991 $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -49,13 +49,13 @@
 #import "bookmarks.h"
 #import "coredialogs.h"
 #import "embeddedwindow.h"
-#import "update.h"
 #import "AppleRemote.h"
 #import "eyetv.h"
 #import "simple_prefs.h"
 
 #import <AddressBook/AddressBook.h>         /* for crashlog send mechanism */
 #import <IOKit/hidsystem/ev_keymap.h>         /* for the media key support */
+#import <Sparkle/Sparkle.h>                 /* we're the update delegate */
 
 /*****************************************************************************
  * Local prototypes.
@@ -110,7 +110,8 @@ void CloseIntf ( vlc_object_t *p_this )
 /*****************************************************************************
  * Run: main loop
  *****************************************************************************/
-jmp_buf jmpbuffer;
+static NSLock * o_appLock = nil;    // controls access to f_appExit
+static int f_appExit = 0;           // set to 1 when application termination signaled
 
 static void Run( intf_thread_t *p_intf )
 {
@@ -131,18 +132,16 @@ static void Run( intf_thread_t *p_intf )
 
     NSAutoreleasePool * o_pool = [[NSAutoreleasePool alloc] init];
 
-    /* Install a jmpbuffer to where we can go back before the NSApp exit
-     * see applicationWillTerminate: */
+    o_appLock = [[NSLock alloc] init];
+
     [VLCApplication sharedApplication];
 
     [[VLCMain sharedInstance] setIntf: p_intf];
     [NSBundle loadNibNamed: @"MainMenu" owner: NSApp];
 
-    /* Install a jmpbuffer to where we can go back before the NSApp exit
-     * see applicationWillTerminate: */
-    if(setjmp(jmpbuffer) == 0)
-        [NSApp run];
-    
+    [NSApp run];
+    [[VLCMain sharedInstance] applicationWillTerminate:nil];
+
     [o_pool release];
 }
 
@@ -159,7 +158,7 @@ static void MsgCallback( msg_cb_data_t *data, msg_item_t *item, unsigned int i )
     int canc = vlc_savecancel();
     NSAutoreleasePool * o_pool = [[NSAutoreleasePool alloc] init];
 
-    /* this may happen from time to time, let's bail out as info would be useless anyway */ 
+    /* this may happen from time to time, let's bail out as info would be useless anyway */
     if( !item->psz_module || !item->psz_msg )
         return;
 
@@ -168,8 +167,8 @@ static void MsgCallback( msg_cb_data_t *data, msg_item_t *item, unsigned int i )
                                 [NSString stringWithUTF8String: item->psz_msg], @"Message",
                                 [NSNumber numberWithInt: item->i_type], @"Type", nil];
 
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"VLCCoreMessageReceived" 
-                                                        object: nil 
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"VLCCoreMessageReceived"
+                                                        object: nil
                                                       userInfo: o_dict];
 
     [o_pool release];
@@ -222,7 +221,7 @@ static int FullscreenChanged( vlc_object_t *p_this, const char *psz_variable,
 }
 
 /*****************************************************************************
- * DialogCallback: Callback triggered by the "dialog-*" variables 
+ * DialogCallback: Callback triggered by the "dialog-*" variables
  * to let the intf display error and interaction dialogs
  *****************************************************************************/
 static int DialogCallback( vlc_object_t *p_this, const char *type, vlc_value_t previous, vlc_value_t value, void *data )
@@ -277,6 +276,37 @@ bool checkProgressPanel (void *priv)
     [o_pool release];
 }
 
+#pragma mark -
+#pragma mark Helpers
+
+input_thread_t *getInput(void)
+{
+    intf_thread_t *p_intf = VLCIntf;
+    if (!p_intf)
+        return NULL;
+    return pl_CurrentInput(p_intf);
+}
+
+vout_thread_t *getVout(void)
+{
+    input_thread_t *p_input = getInput();
+    if (!p_input)
+        return NULL;
+    vout_thread_t *p_vout = input_GetVout(p_input);
+    vlc_object_release(p_input);
+    return p_vout;
+}
+
+aout_instance_t *getAout(void)
+{
+    input_thread_t *p_input = getInput();
+    if (!p_input)
+        return NULL;
+    aout_instance_t *p_aout = input_GetAout(p_input);
+    vlc_object_release(p_input);
+    return p_aout;
+}
+
 #pragma mark -
 #pragma mark Private
 
@@ -301,11 +331,11 @@ static VLCMain *_o_sharedMainInstance = nil;
 
 - (id)init
 {
-    if( _o_sharedMainInstance) 
+    if( _o_sharedMainInstance)
     {
         [self dealloc];
         return _o_sharedMainInstance;
-    } 
+    }
     else
         _o_sharedMainInstance = [super init];
 
@@ -315,7 +345,7 @@ static VLCMain *_o_sharedMainInstance = nil;
     o_msg_arr = [[NSMutableArray arrayWithCapacity: 600] retain];
     /* subscribe to LibVLC's debug messages as early as possible (for us) */
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(libvlcMessageReceived:) name: @"VLCCoreMessageReceived" object: nil];
-    
+
     o_about = [[VLAboutBox alloc] init];
     o_prefs = nil;
     o_open = [[VLCOpen alloc] init];
@@ -325,16 +355,9 @@ static VLCMain *_o_sharedMainInstance = nil;
     o_embedded_list = [[VLCEmbeddedList alloc] init];
     o_coredialogs = [[VLCCoreDialogProvider alloc] init];
     o_info = [[VLCInfo alloc] init];
-#ifdef UPDATE_CHECK
-    o_update = [[VLCUpdate alloc] init];
-#endif
 
     i_lastShownVolume = -1;
 
-    o_remote = [[AppleRemote alloc] init];
-    [o_remote setClickCountEnabledButtons: kRemoteButtonPlay];
-    [o_remote setDelegate: _o_sharedMainInstance];
-
     o_eyetv = [[VLCEyeTVController alloc] init];
 
     /* announce our launch to a potential eyetv plugin */
@@ -366,30 +389,6 @@ static VLCMain *_o_sharedMainInstance = nil;
        because VLCMain is the owner */
     if( nib_main_loaded ) return;
 
-    /* check whether the user runs a valid version of OS X */
-    if( MACOS_VERSION < 10.5f )
-    {
-        NSAlert *ourAlert;
-        int i_returnValue;
-        NSString *o_blabla;
-        if( MACOS_VERSION == 10.4f )
-            o_blabla = _NS("VLC's last release for your OS is the 0.9 series." );
-        else if( MACOS_VERSION == 10.3f )
-            o_blabla = _NS("VLC's last release for your OS is VLC 0.8.6i, which is prone to known security issues." );
-        else // 10.2 and 10.1, still 3% of the OS X market share
-            o_blabla = _NS("VLC's last release for your OS is VLC 0.7.2, which is highly out of date and prone to " \
-                         "known security issues. We recommend you to update your Mac to a modern version of Mac OS X.");
-        ourAlert = [NSAlert alertWithMessageText: _NS("Your version of Mac OS X is no longer supported")
-                                   defaultButton: _NS("Quit")
-                                 alternateButton: NULL
-                                     otherButton: NULL
-                       informativeTextWithFormat: _NS("VLC media player %s requires Mac OS X 10.5 or higher.\n\n%@"), VLC_Version(), o_blabla];
-        [ourAlert setAlertStyle: NSCriticalAlertStyle];
-        i_returnValue = [ourAlert runModal];
-        [NSApp performSelectorOnMainThread: @selector(terminate:) withObject:nil waitUntilDone:NO];
-        return;
-    }
-
     [self initStrings];
 
     [o_window setExcludedFromWindowsMenu: YES];
@@ -420,6 +419,9 @@ static VLCMain *_o_sharedMainInstance = nil;
     i_key = config_GetInt( p_intf, "key-slower" );
     [o_mi_slower setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
     [o_mi_slower setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-rate-normal" );
+    [o_mi_normalSpeed setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_normalSpeed setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
     i_key = config_GetInt( p_intf, "key-prev" );
     [o_mi_previous setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
     [o_mi_previous setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
@@ -453,12 +455,24 @@ static VLCMain *_o_sharedMainInstance = nil;
     i_key = config_GetInt( p_intf, "key-vol-mute" );
     [o_mi_mute setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
     [o_mi_mute setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
-    i_key = config_GetInt( p_intf, "key-fullscreen" );
+    i_key = config_GetInt( p_intf, "key-toggle-fullscreen" );
     [o_mi_fullscreen setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
     [o_mi_fullscreen setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
     i_key = config_GetInt( p_intf, "key-snapshot" );
     [o_mi_snapshot setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
     [o_mi_snapshot setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-random" );
+    [o_mi_random setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_random setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-zoom-half" );
+    [o_mi_half_window setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_half_window setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-zoom-original" );
+    [o_mi_normal_window setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_normal_window setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-zoom-double" );
+    [o_mi_double_window setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_double_window setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
 
     var_Create( p_intf, "intf-change", VLC_VAR_BOOL );
 
@@ -466,7 +480,7 @@ static VLCMain *_o_sharedMainInstance = nil;
     [o_volumeslider setEnabled: YES];
     [self manageVolumeSlider];
     [o_window setDelegate: self];
- 
+
     b_restore_size = false;
 
     // Set that here as IB seems to be buggy
@@ -494,19 +508,16 @@ static VLCMain *_o_sharedMainInstance = nil;
 
     o_size_with_playlist = [o_window contentRectForFrameRect:[o_window frame]].size;
 
-    p_playlist = pl_Hold( p_intf );
+    p_playlist = pl_Get( p_intf );
 
-    var_Create( p_playlist, "fullscreen", VLC_VAR_BOOL | VLC_VAR_DOINHERIT);
     val.b_bool = false;
 
     var_AddCallback( p_playlist, "fullscreen", FullscreenChanged, self);
     var_AddCallback( p_intf->p_libvlc, "intf-show", ShowController, self);
 
-    pl_Release( p_intf );
-
     /* load our Core Dialogs nib */
     nib_coredialogs_loaded = [NSBundle loadNibNamed:@"CoreDialogs" owner: NSApp];
-    
+
     /* subscribe to various interactive dialogues */
     var_Create( p_intf, "dialog-error", VLC_VAR_ADDRESS );
     var_AddCallback( p_intf, "dialog-error", DialogCallback, self );
@@ -530,13 +541,18 @@ static VLCMain *_o_sharedMainInstance = nil;
 
     /* take care of tint changes during runtime */
     o_img_play = [NSImage imageNamed: @"play"];
-    o_img_pause = [NSImage imageNamed: @"pause"];    
+    o_img_pause = [NSImage imageNamed: @"pause"];
     [self controlTintChanged];
     [[NSNotificationCenter defaultCenter] addObserver: self
                                              selector: @selector( controlTintChanged )
                                                  name: NSControlTintDidChangeNotification
                                                object: nil];
 
+    /* init Apple Remote support */
+    o_remote = [[AppleRemote alloc] init];
+    [o_remote setClickCountEnabledButtons: kRemoteButtonPlay];
+    [o_remote setDelegate: _o_sharedMainInstance];
+
     /* yeah, we are done */
     nib_main_loaded = TRUE;
 }
@@ -565,15 +581,6 @@ static VLCMain *_o_sharedMainInstance = nil;
 
     [self _removeOldPreferences];
 
-#ifdef UPDATE_CHECK
-    /* Check for update silently on startup */
-    if( !nib_update_loaded )
-        nib_update_loaded = [NSBundle loadNibNamed:@"Update" owner: NSApp];
-
-    if([o_update shouldCheckForUpdate])
-        [NSThread detachNewThreadSelector:@selector(checkForUpdate) toTarget:o_update withObject:nil];
-#endif
-
     /* Handle sleep notification */
     [[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self selector:@selector(computerWillSleep:)
            name:NSWorkspaceWillSleepNotification object:nil];
@@ -640,6 +647,7 @@ static VLCMain *_o_sharedMainInstance = nil;
     [o_mi_stop setTitle: _NS("Stop")];
     [o_mi_faster setTitle: _NS("Faster")];
     [o_mi_slower setTitle: _NS("Slower")];
+    [o_mi_normalSpeed setTitle: _NS("Normal rate")];
     [o_mi_previous setTitle: _NS("Previous")];
     [o_mi_next setTitle: _NS("Next")];
     [o_mi_random setTitle: _NS("Random")];
@@ -703,6 +711,7 @@ static VLCMain *_o_sharedMainInstance = nil;
     [o_mu_window setTitle: _NS("Window")];
     [o_mi_minimize setTitle: _NS("Minimize Window")];
     [o_mi_close_window setTitle: _NS("Close Window")];
+    [o_mi_player setTitle: _NS("Player...")];
     [o_mi_controller setTitle: _NS("Controller...")];
     [o_mi_equalizer setTitle: _NS("Equalizer...")];
     [o_mi_extended setTitle: _NS("Extended Controls...")];
@@ -729,7 +738,7 @@ static VLCMain *_o_sharedMainInstance = nil;
     [o_dmi_next setTitle: _NS("Next")];
     [o_dmi_previous setTitle: _NS("Previous")];
     [o_dmi_mute setTitle: _NS("Mute")];
- 
+
     /* vout menu */
     [o_vmi_play setTitle: _NS("Play")];
     [o_vmi_stop setTitle: _NS("Stop")];
@@ -774,20 +783,28 @@ static VLCMain *_o_sharedMainInstance = nil;
     playlist_t * p_playlist;
     vout_thread_t * p_vout;
     int returnedValue = 0;
- 
-    if( !p_intf ) return;
+
+    if( !p_intf )
+        return;
+
+    // don't allow a double termination call. If the user has
+    // already invoked the quit then simply return this time.
+    int isTerminating = false;
+
+    [o_appLock lock];
+    isTerminating = (f_appExit++ > 0 ? 1 : 0);
+    [o_appLock unlock];
+
+    if (isTerminating)
+        return;
 
     msg_Dbg( p_intf, "Terminating" );
 
-    /* Make sure the manage_thread won't call -terminate: again */
-    pthread_cancel( manage_thread );
+    pthread_join( manage_thread, NULL );
 
     /* Make sure the intf object is getting killed */
     vlc_object_kill( p_intf );
 
-    /* Make sure our manage_thread ends */
-    pthread_join( manage_thread, NULL );
-
     /* Make sure the interfaceTimer is destroyed */
     [interfaceTimer invalidate];
     [interfaceTimer release];
@@ -844,13 +861,9 @@ static VLCMain *_o_sharedMainInstance = nil;
     if( nib_wizard_loaded )
         [o_wizard release];
 
-#ifdef UPDATE_CHECK
-    [o_update release]; 
-#endif
-
     [crashLogURLConnection cancel];
     [crashLogURLConnection release];
- 
+
     [o_embedded_list release];
     [o_coredialogs release];
     [o_eyetv release];
@@ -876,18 +889,24 @@ static VLCMain *_o_sharedMainInstance = nil;
 
     /* Kill the playlist, so that it doesn't accept new request
      * such as the play request from vlc.c (we are a blocking interface). */
-    p_playlist = pl_Hold( p_intf );
+    p_playlist = pl_Get( p_intf );
     vlc_object_kill( p_playlist );
-    pl_Release( p_intf );
-
     libvlc_Quit( p_intf->p_libvlc );
 
     [self setIntf:nil];
+}
 
-    /* Go back to Run() and make libvlc exit properly */
-    if( jmpbuffer )
-        longjmp( jmpbuffer, 1 );
-    /* not reached */
+#pragma mark -
+#pragma mark Sparkle delegate
+/* received directly before the update gets installed, so let's shut down a bit */
+- (void)updater:(SUUpdater *)updater willInstallUpdate:(SUAppcastItem *)update
+{
+    [o_remote stopListening: self];
+    var_SetInteger( p_intf->p_libvlc, "key-action", ACTIONID_STOP );
+
+    /* Close the window directly, because we do know that there
+     * won't be anymore video. It's currently waiting a bit. */
+    [[[o_controls voutView] window] orderOut:self];
 }
 
 #pragma mark -
@@ -956,15 +975,15 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
 - (void)controlTintChanged
 {
     BOOL b_playing = NO;
-    
+
     if( [o_btn_play alternateImage] == o_img_play_pressed )
         b_playing = YES;
-    
+
     if( [NSColor currentControlTint] == NSGraphiteControlTint )
     {
         o_img_play_pressed = [NSImage imageNamed: @"play_graphite"];
         o_img_pause_pressed = [NSImage imageNamed: @"pause_graphite"];
-        
+
         [o_btn_prev setAlternateImage: [NSImage imageNamed: @"previous_graphite"]];
         [o_btn_rewind setAlternateImage: [NSImage imageNamed: @"skip_previous_graphite"]];
         [o_btn_stop setAlternateImage: [NSImage imageNamed: @"stop_graphite"]];
@@ -978,7 +997,7 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
     {
         o_img_play_pressed = [NSImage imageNamed: @"play_blue"];
         o_img_pause_pressed = [NSImage imageNamed: @"pause_blue"];
-        
+
         [o_btn_prev setAlternateImage: [NSImage imageNamed: @"previous_blue"]];
         [o_btn_rewind setAlternateImage: [NSImage imageNamed: @"skip_previous_blue"]];
         [o_btn_stop setAlternateImage: [NSImage imageNamed: @"stop_blue"]];
@@ -988,7 +1007,7 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
         [o_btn_playlist setAlternateImage: [NSImage imageNamed: @"playlistdrawer_blue"]];
         [o_btn_equalizer setAlternateImage: [NSImage imageNamed: @"equalizerdrawer_blue"]];
     }
-    
+
     if( b_playing )
         [o_btn_play setAlternateImage: o_img_play_pressed];
     else
@@ -1000,7 +1019,8 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
 - (void)applicationDidBecomeActive:(NSNotification *)aNotification
 {
     if( !p_intf ) return;
-    [o_remote startListening: self];
+	if( config_GetInt( p_intf, "macosx-appleremote" ) == YES )
+		[o_remote startListening: self];
 }
 - (void)applicationDidResignActive:(NSNotification *)aNotification
 {
@@ -1014,9 +1034,7 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
     /* Pause */
     if( p_intf && p_intf->p_sys->i_play_status == PLAYING_S )
     {
-        vlc_value_t val;
-        val.i_int = config_GetInt( p_intf, "key-play-pause" );
-        var_Set( p_intf->p_libvlc, "key-pressed", val );
+        var_SetInteger( p_intf->p_libvlc, "key-action", ACTIONID_PLAY_PAUSE );
     }
 }
 
@@ -1037,7 +1055,7 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
 
 /* When user click in the Dock icon our double click in the finder */
 - (BOOL)applicationShouldHandleReopen:(NSApplication *)theApplication hasVisibleWindows:(BOOL)hasVisibleWindows
-{    
+{
     if(!hasVisibleWindows)
         [o_window makeKeyAndOrderFront:self];
 
@@ -1085,6 +1103,12 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
 {
     switch( buttonIdentifier )
     {
+        case k2009RemoteButtonFullscreen:
+            [o_controls toogleFullscreen:self];
+            break;
+        case k2009RemoteButtonPlay:
+            [o_controls play:self];
+            break;
         case kRemoteButtonPlay:
             if(count >= 2) {
                 [o_controls toogleFullscreen:self];
@@ -1260,8 +1284,6 @@ static struct
     { NSCarriageReturnCharacter, KEY_ENTER },
     { NSEnterCharacter, KEY_ENTER },
     { NSBackspaceCharacter, KEY_BACKSPACE },
-    { (unichar) ' ', KEY_SPACE },
-    { (unichar) 0x1b, KEY_ESC },
     {0,0}
 };
 
@@ -1520,8 +1542,6 @@ static void manage_cleanup( void * args )
     var_DelCallback( p_playlist, "playlist-item-append", PlaylistChanged, self );
     var_DelCallback( p_playlist, "playlist-item-deleted", PlaylistChanged, self );
 
-    pl_Release( p_intf );
-
     if( p_input ) vlc_object_release( p_input );
 }
 
@@ -1534,7 +1554,7 @@ static void manage_cleanup( void * args )
 
     vlc_thread_set_priority( p_intf, VLC_THREAD_PRIORITY_LOW );
 
-    p_playlist = pl_Hold( p_intf );
+    p_playlist = pl_Get( p_intf );
 
     var_AddCallback( p_playlist, "item-current", PlaylistChanged, self );
     var_AddCallback( p_playlist, "intf-change", PlaylistChanged, self );
@@ -1545,7 +1565,8 @@ static void manage_cleanup( void * args )
     struct manage_cleanup_stack stack = { p_intf, &p_input, p_playlist, self };
     pthread_cleanup_push(manage_cleanup, &stack);
 
-    while( true )
+    bool exitLoop = false;
+    while( !exitLoop )
     {
         NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
 
@@ -1580,20 +1601,21 @@ static void manage_cleanup( void * args )
         msleep( INTF_IDLE_SLEEP );
 
         [pool release];
+
+        [o_appLock lock];
+        exitLoop = (f_appExit != 0 ? true : false);
+        [o_appLock unlock];
     }
 
     pthread_cleanup_pop(1);
-
-    msg_Dbg( p_intf, "Killing the Mac OS X module" );
-
-    /* We are dead, terminate */
-    [NSApp performSelectorOnMainThread: @selector(terminate:) withObject:nil waitUntilDone:NO];
 }
 
 - (void)manageVolumeSlider
 {
     audio_volume_t i_volume;
-    aout_VolumeGet( p_intf, &i_volume );
+    playlist_t * p_playlist = pl_Get( p_intf );
+
+    aout_VolumeGet( p_playlist, &i_volume );
 
     if( i_volume != i_lastShownVolume )
     {
@@ -1620,13 +1642,12 @@ static void manage_cleanup( void * args )
          * the playlist or the selection is empty */
         if( [self isPlaylistCollapsed] == YES )
         {
-            playlist_t * p_playlist = pl_Hold( p_intf );
+            playlist_t * p_playlist = pl_Get( p_intf );
             PL_LOCK;
             playlist_item_t * p_item = playlist_CurrentPlayingItem( p_playlist );
             PL_UNLOCK;
             if( p_item )
                 [[self info] updatePanelWithItem: p_item->p_input];
-            pl_Release( p_intf );
         }
     }
     if( p_intf->p_sys->b_intf_update )
@@ -1637,7 +1658,7 @@ static void manage_cleanup( void * args )
         bool b_seekable = false;
         bool b_chapters = false;
 
-        playlist_t * p_playlist = pl_Hold( p_intf );
+        playlist_t * p_playlist = pl_Get( p_intf );
 
         PL_LOCK;
         b_plmul = playlist_CurrentSize( p_playlist ) > 1;
@@ -1646,7 +1667,7 @@ static void manage_cleanup( void * args )
         p_input = playlist_CurrentInput( p_playlist );
 
         bool b_buffering = NO;
-    
+
         if( ( b_input = ( p_input != NULL ) ) )
         {
             /* seekable streams */
@@ -1667,7 +1688,6 @@ static void manage_cleanup( void * args )
             //b_chapters = p_input->stream.i_area_nb > 1;
             vlc_object_release( p_input );
         }
-        pl_Release( p_intf );
 
         if( b_buffering )
         {
@@ -1682,10 +1702,13 @@ static void manage_cleanup( void * args )
         }
 
         [o_btn_stop setEnabled: b_input];
+        [o_embedded_window setStop: b_input];
         [o_btn_ff setEnabled: b_seekable];
         [o_btn_rewind setEnabled: b_seekable];
         [o_btn_prev setEnabled: (b_plmul || b_chapters)];
+        [o_embedded_window setPrev: (b_plmul || b_chapters)];
         [o_btn_next setEnabled: (b_plmul || b_chapters)];
+        [o_embedded_window setNext: (b_plmul || b_chapters)];
 
         [o_timeslider setFloatValue: 0.0];
         [o_timeslider setEnabled: b_seekable];
@@ -1694,9 +1717,10 @@ static void manage_cleanup( void * args )
         [[[self controls] fspanel] setSeekable: b_seekable];
 
         [o_embedded_window setSeekable: b_seekable];
+        [o_embedded_window setTime:@"00:00" position:0.0];
 
         p_intf->p_sys->b_current_title_update = true;
-        
+
         p_intf->p_sys->b_intf_update = false;
     }
 
@@ -1748,7 +1772,7 @@ static void manage_cleanup( void * args )
             [[[self controls] fspanel] setStreamTitle: aString];
 
             [[o_controls voutView] updateTitle];
- 
+
             [o_playlist updateRowSelection];
 
             p_intf->p_sys->b_current_title_update = FALSE;
@@ -1814,6 +1838,8 @@ static void manage_cleanup( void * args )
         i_volume_step = config_GetInt( p_intf->p_libvlc, "volume-step" );
         [o_volumeslider setFloatValue: (float)i_lastShownVolume / i_volume_step];
         [o_volumeslider setEnabled: TRUE];
+        [o_embedded_window setVolumeSlider: (float)i_lastShownVolume / i_volume_step];
+        [o_embedded_window setVolumeEnabled: TRUE];
         [[[self controls] fspanel] setVolumeLevel: (float)i_lastShownVolume / i_volume_step];
         p_intf->p_sys->b_mute = ( i_lastShownVolume == 0 );
         p_intf->p_sys->b_volume_update = FALSE;
@@ -1837,7 +1863,7 @@ end:
 
 - (void)setupMenus
 {
-    playlist_t * p_playlist = pl_Hold( p_intf );
+    playlist_t * p_playlist = pl_Get( p_intf );
     input_thread_t * p_input = playlist_CurrentInput( p_playlist );
     if( p_input != NULL )
     {
@@ -1893,7 +1919,7 @@ end:
                 var: "video-device" selector: @selector(toggleVar:)];
 
             [o_controls setupVarMenuItem: o_mi_deinterlace target: (vlc_object_t *)p_vout
-                var: "deinterlace" selector: @selector(toggleVar:)];
+                var: "deinterlace-mode" selector: @selector(toggleVar:)];
 
 #if 1
            [o_controls setupVarMenuItem: o_mi_ffmpeg_pp target:
@@ -1905,18 +1931,15 @@ end:
         }
         vlc_object_release( p_input );
     }
-    pl_Release( p_intf );
 }
 
 - (void)refreshVoutDeviceMenu:(NSNotification *)o_notification
 {
-    int x,y = 0;
-    vout_thread_t * p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
-                                              FIND_ANYWHERE );
- 
-    if(! p_vout )
+    int x, y = 0;
+    vout_thread_t * p_vout = getVout();
+    if( !p_vout )
         return;
- 
+
     /* clean the menu before adding new entries */
     if( [o_mi_screen hasSubmenu] )
     {
@@ -1942,11 +1965,12 @@ end:
     else
         i_end_scroll = -1;
     [o_scrollfield setStringValue: o_string];
+    [o_embedded_window setScrollString: o_string];
 }
 
 - (void)resetScrollField
 {
-    playlist_t * p_playlist = pl_Hold( p_intf );
+    playlist_t * p_playlist = pl_Get( p_intf );
     input_thread_t * p_input = playlist_CurrentInput( p_playlist );
 
     i_end_scroll = -1;
@@ -1963,10 +1987,8 @@ end:
         [self setScrollField: o_temp stopAfter:-1];
         [[[self controls] fspanel] setStreamTitle: o_temp];
         vlc_object_release( p_input );
-        pl_Release( p_intf );
         return;
     }
-    pl_Release( p_intf );
     [self setScrollField: _NS("VLC media player") stopAfter:-1];
 }
 
@@ -2030,7 +2052,7 @@ end:
         default:
             return;
     }
-    p_playlist = pl_Hold( p_intf );
+    p_playlist = pl_Get( p_intf );
     p_input = playlist_CurrentInput( p_playlist );
     if( p_input != NULL )
     {
@@ -2058,14 +2080,13 @@ end:
         [o_embedded_window setTime: o_time position: f_updated];
         vlc_object_release( p_input );
     }
-    pl_Release( p_intf );
 }
 
 - (IBAction)timeFieldWasClicked:(id)sender
 {
     b_time_remaining = !b_time_remaining;
 }
-    
+
 
 #pragma mark -
 #pragma mark Recent Items
@@ -2177,7 +2198,7 @@ end:
         nib_wizard_loaded = [NSBundle loadNibNamed:@"Wizard" owner: NSApp];
         [o_wizard initStrings];
     }
- 
+
     if( !nib_bookmarks_loaded )
         nib_bookmarks_loaded = [NSBundle loadNibNamed:@"Bookmarks" owner: NSApp];
 
@@ -2196,21 +2217,6 @@ end:
     [o_sprefs showSimplePrefs];
 }
 
-#pragma mark -
-#pragma mark Update
-
-- (IBAction)checkForUpdate:(id)sender
-{
-#ifdef UPDATE_CHECK
-    if( !nib_update_loaded )
-        nib_update_loaded = [NSBundle loadNibNamed:@"Update" owner: NSApp];
-    [o_update showUpdateWindow];
-#else
-    msg_Err( VLCIntf, "Update checker wasn't enabled in this build" );
-    dialog_FatalWait( VLCIntf, _("Update check failed"), _("Checking for updates was not enabled in this build.") );
-#endif
-}
-
 #pragma mark -
 #pragma mark Help and Docs
 
@@ -2229,7 +2235,7 @@ end:
 
     [o_about showGPL: sender];
 }
-    
+
 - (IBAction)viewHelp:(id)sender
 {
     if( !nib_about_loaded )
@@ -2348,7 +2354,7 @@ end:
 
             // Dooh. ugly.
             if( year < [[compo objectAtIndex:0] intValue] ||
-                (year ==[[compo objectAtIndex:0] intValue] && 
+                (year ==[[compo objectAtIndex:0] intValue] &&
                  (month < [[compo objectAtIndex:1] intValue] ||
                   (month ==[[compo objectAtIndex:1] intValue] &&
                    (day   < [[compo objectAtIndex:2] intValue] ||
@@ -2425,7 +2431,7 @@ end:
     int version = [[NSUserDefaults standardUserDefaults] integerForKey:kVLCPreferencesVersion];
     if( version >= kCurrentPreferencesVersion ) return;
 
-    NSArray *libraries = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, 
+    NSArray *libraries = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,
         NSUserDomainMask, YES);
     if( !libraries || [libraries count] == 0) return;
     NSString * preferences = [[libraries objectAtIndex:0] stringByAppendingPathComponent:@"Preferences"];
@@ -2487,7 +2493,7 @@ end:
 {
     if(! nib_info_loaded )
         nib_info_loaded = [NSBundle loadNibNamed:@"MediaInfo" owner: NSApp];
-    
+
     [o_info initPanel];
 }
 
@@ -2568,7 +2574,7 @@ end:
 - (IBAction)saveDebugLog:(id)sender
 {
     NSOpenPanel * saveFolderPanel = [[NSSavePanel alloc] init];
-    
+
     [saveFolderPanel setCanChooseDirectories: NO];
     [saveFolderPanel setCanChooseFiles: YES];
     [saveFolderPanel setCanSelectHiddenExtension: NO];
@@ -2607,7 +2613,7 @@ end:
             o_rect.size.height = o_size_with_playlist.height;
         else
             o_rect.size.height = 500.;
- 
+
         if( o_size_with_playlist.width >= [o_window contentMinSize].width )
             o_rect.size.width = o_size_with_playlist.width;
         else
@@ -2868,7 +2874,7 @@ end:
             {
                 if( keyState == 0 && keyRepeat == 0 )
                 {
-                        var_SetInteger( VLCIntf->p_libvlc, "key-action", ACTIONID_NEXT );
+                    var_SetInteger( VLCIntf->p_libvlc, "key-action", ACTIONID_NEXT );
                 }
                 else if( keyRepeat == 1 )
                 {
