@@ -2,7 +2,7 @@
  * Messages.cpp : Information about an item
  ****************************************************************************
  * Copyright (C) 2006-2007 the VideoLAN team
- * $Id: 11a9fef9d6059463514e19e26ea21dffaf84ef1a $
+ * $Id: 3558aa2581753bf3b909dca782c26f6fa325ec25 $
  *
  * Authors: Jean-Baptiste Kempf <jb (at) videolan.org>
  *
@@ -38,11 +38,11 @@
 #include <QTreeWidgetItem>
 #include <QHeaderView>
 #include <QMutex>
+#include <QLineEdit>
+#include <QPushButton>
 
 #include <assert.h>
 
-MessagesDialog *MessagesDialog::instance = NULL;
-
 enum {
     MsgEvent_Type = QEvent::User + MsgEventType + 1,
 };
@@ -73,6 +73,7 @@ MessagesDialog::MessagesDialog( intf_thread_t *_p_intf)
                : QVLCFrame( _p_intf )
 {
     setWindowTitle( qtr( "Messages" ) );
+    setWindowRole( "vlc-messages" );
 
     /* General widgets */
     QGridLayout *mainLayout = new QGridLayout( this );
@@ -114,25 +115,38 @@ MessagesDialog::MessagesDialog( intf_thread_t *_p_intf)
 
     verbosityBox = new QSpinBox();
     verbosityBox->setRange( 0, 2 );
-    verbosityBox->setValue( config_GetInt( p_intf, "verbose" ) );
+    verbosityBox->setValue( var_InheritInteger( p_intf, "verbose" ) );
     verbosityBox->setWrapping( true );
     verbosityBox->setMaximumWidth( 50 );
 
     verbosityLabel = new QLabel( qtr( "Verbosity Level" ) );
 
+    vbobjectsEdit = new QLineEdit();
+
+    vbobjectsEdit->setMaximumWidth( 100 );
+    vbobjectsEdit->setText(config_GetPsz( p_intf, "verbose-objects"));
+    vbobjectsEdit->setToolTip( "verbose-objects usage: \n"
+                            "--verbose-objects=+printthatobject,-dontprintthatone\n"
+                            "(keyword 'all' to applies to all objects)");
+
+    vbobjectsLabel =  new QLabel( qtr( "Message filter" ) );
+
     mainLayout->addWidget( mainTab, 0, 0, 1, 0 );
     mainLayout->addWidget( verbosityLabel, 1, 0, 1, 1 );
     mainLayout->addWidget( verbosityBox, 1, 1 );
-    mainLayout->setColumnStretch( 2, 10 );
-    mainLayout->addWidget( saveLogButton, 1, 3 );
-    mainLayout->addWidget( clearUpdateButton, 1, 4 );
-    mainLayout->addWidget( closeButton, 1, 5 );
+    mainLayout->addWidget( vbobjectsLabel, 1, 2, 1, 1 );
+    mainLayout->addWidget( vbobjectsEdit, 1, 3 );
+    mainLayout->setColumnStretch( 4, 10 );
+    mainLayout->addWidget( saveLogButton, 1, 5 );
+    mainLayout->addWidget( clearUpdateButton, 1, 6 );
+    mainLayout->addWidget( closeButton, 1, 7 );
 
     BUTTONACT( closeButton, hide() );
     BUTTONACT( clearUpdateButton, clearOrUpdate() );
     BUTTONACT( saveLogButton, save() );
     CONNECT( mainTab, currentChanged( int ),
              this, updateTab( int ) );
+    CONNECT(vbobjectsEdit, editingFinished(), this, updateConfig());
 
     /* General action */
     readSettings( "Messages", QSize( 600, 450 ) );
@@ -158,6 +172,8 @@ void MessagesDialog::updateTab( int index )
     {
         verbosityLabel->hide();
         verbosityBox->hide();
+        vbobjectsLabel->hide();
+        vbobjectsEdit->hide();
         clearUpdateButton->setText( qtr( "&Update" ) );
         saveLogButton->hide();
         updateTree();
@@ -167,11 +183,36 @@ void MessagesDialog::updateTab( int index )
     {
         verbosityLabel->show();
         verbosityBox->show();
+        vbobjectsLabel->show();
+        vbobjectsEdit->show();
         clearUpdateButton->setText( qtr( "&Clear" ) );
         saveLogButton->show();
     }
 }
 
+void MessagesDialog::updateConfig()
+{
+    config_PutPsz(p_intf, "verbose-objects", qtu(vbobjectsEdit->text()));
+    //vbobjectsEdit->setText("vbEdit changed!");
+
+    char * psz_verbose_objects = strdup(qtu(vbobjectsEdit->text()));
+    msg_EnableObjectPrinting(p_intf, "all");
+    if( psz_verbose_objects )
+    {
+        char * psz_object, * iter =  psz_verbose_objects;
+        while( (psz_object = strsep( &iter, "," )) )
+        {
+            switch( psz_object[0] )
+            {
+                printf("%s\n", psz_object+1);
+                case '+': msg_EnableObjectPrinting(p_intf, psz_object+1); break;
+                case '-': msg_DisableObjectPrinting(p_intf, psz_object+1); break;
+             }
+        }
+        free( psz_verbose_objects );
+    }
+}
+
 void MessagesDialog::sinkMessage( msg_item_t *item )
 {
     if ((item->i_type == VLC_MSG_WARN && verbosityBox->value() < 1)
@@ -245,7 +286,7 @@ bool MessagesDialog::save()
 {
     QString saveLogFileName = QFileDialog::getSaveFileName(
             this, qtr( "Save log file as..." ),
-            qfu( config_GetHomeDir() ),
+            QVLCUserDir( VLC_DOCUMENTS_DIR ),
             qtr( "Texts / Logs (*.log *.txt);; All (*.*) ") );
 
     if( !saveLogFileName.isNull() )
@@ -277,10 +318,14 @@ void MessagesDialog::buildTree( QTreeWidgetItem *parentItem,
     else
         item = new QTreeWidgetItem( modulesTree );
 
-    if( p_obj->psz_object_name )
+    char *name = vlc_object_get_name( p_obj );
+    if( name != NULL )
+    {
         item->setText( 0, qfu( p_obj->psz_object_type ) + " \"" +
-                       qfu( p_obj->psz_object_name ) + "\" (" +
+                       qfu( name ) + "\" (" +
                        QString::number((uintptr_t)p_obj) + ")" );
+        free( name );
+    }
     else
         item->setText( 0, qfu( p_obj->psz_object_type ) + " (" +
                        QString::number((uintptr_t)p_obj) + ")" );
