@@ -2,7 +2,7 @@
  * input_manager.cpp : Manage an input and interact with its GUI elements
  ****************************************************************************
  * Copyright (C) 2006-2008 the VideoLAN team
- * $Id: d032f55b19d2e55dbf8a58fc3d9d70e9470c861e $
+ * $Id: 7717cd2185c03c7b20fe50a5e92f9c1844bfc43f $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Ilkka Ollakka  <ileoo@videolan.org>
@@ -37,10 +37,26 @@
 
 static int ItemChanged( vlc_object_t *, const char *,
                         vlc_value_t, vlc_value_t, void * );
+static int LeafToParent( vlc_object_t *, const char *,
+                        vlc_value_t, vlc_value_t, void * );
 static int PLItemChanged( vlc_object_t *, const char *,
                         vlc_value_t, vlc_value_t, void * );
+static int PLItemAppended( vlc_object_t *, const char *,
+                        vlc_value_t, vlc_value_t, void * );
+static int PLItemRemoved( vlc_object_t *, const char *,
+                        vlc_value_t, vlc_value_t, void * );
 static int VolumeChanged( vlc_object_t *, const char *,
                         vlc_value_t, vlc_value_t, void * );
+static int SoundMuteChanged( vlc_object_t *, const char *,
+                        vlc_value_t, vlc_value_t, void * );
+
+static int RandomChanged( vlc_object_t *, const char *,
+                        vlc_value_t, vlc_value_t, void * );
+static int LoopChanged( vlc_object_t *, const char *,
+                        vlc_value_t, vlc_value_t, void * );
+static int RepeatChanged( vlc_object_t *, const char *,
+                        vlc_value_t, vlc_value_t, void * );
+
 
 static int InputEvent( vlc_object_t *, const char *,
                        vlc_value_t, vlc_value_t, void * );
@@ -62,8 +78,9 @@ InputManager::InputManager( QObject *parent, intf_thread_t *_p_intf) :
     oldName      = "";
     artUrl       = "";
     p_input      = NULL;
-    i_rate       = 0;
-    i_input_id   = 0;
+    p_input_vbi  = NULL;
+    f_rate       = 0.;
+    p_item       = NULL;
     b_video      = false;
     timeA        = 0;
     timeB        = 0;
@@ -86,20 +103,23 @@ void InputManager::setInput( input_thread_t *_p_input )
     {
         msg_Dbg( p_intf, "IM: Setting an input" );
         vlc_object_hold( p_input );
-        emit statusChanged( PLAYING_S );
+        addCallbacks();
+        UpdateStatus();
         UpdateName();
         UpdateArt();
         UpdateTeletext();
         UpdateNavigation();
         UpdateVout();
-        addCallbacks();
-        i_input_id = input_GetItem( p_input )->i_id;
+
+        p_item = input_GetItem( p_input );
+        emit rateChanged( var_GetFloat( p_input, "rate" ) );
     }
     else
     {
         p_input = NULL;
-        i_input_id = 0;
-        emit rateChanged( INPUT_RATE_DEFAULT );
+        p_item = NULL;
+        assert( !p_input_vbi );
+        emit rateChanged( var_InheritFloat( p_intf, "rate" ) );
     }
 }
 
@@ -113,18 +133,25 @@ void InputManager::delInput()
 
     delCallbacks();
     i_old_playing_status = END_S;
-    i_input_id           = 0;
+    p_item               = NULL;
     oldName              = "";
     artUrl               = "";
     b_video              = false;
     timeA                = 0;
     timeB                = 0;
+    f_rate               = 0. ;
+
+    if( p_input_vbi )
+    {
+        vlc_object_release( p_input_vbi );
+        p_input_vbi = NULL;
+    }
 
     vlc_object_release( p_input );
     p_input = NULL;
 
     emit positionUpdated( -1.0, 0 ,0 );
-    emit rateChanged( INPUT_RATE_DEFAULT ); /* TODO: Do we want this ? */
+    emit rateChanged( var_InheritFloat( p_intf, "rate" ) );
     emit nameChanged( "" );
     emit chapterChanged( 0 );
     emit titleChanged( 0 );
@@ -138,10 +165,12 @@ void InputManager::delInput()
     /* Reset all InfoPanels but stats */
     emit artChanged( NULL );
     emit infoChanged( NULL );
-    emit metaChanged( (input_item_t *)NULL );
+    emit currentMetaChanged( (input_item_t *)NULL );
 
     emit encryptionChanged( false );
     emit recordingStateChanged( false );
+
+    emit cachingChanged( 1 );
 }
 
 /* Convert the event from the callbacks in actions */
@@ -151,18 +180,11 @@ void InputManager::customEvent( QEvent *event )
     IMEvent *ple = static_cast<IMEvent *>(event);
 
     if( i_type == ItemChanged_Type )
-        UpdateMeta( ple->i_id );
+        UpdateMeta( ple->p_item );
 
     if( !hasInput() )
         return;
 
-#ifndef NDEBUG
-    if( i_type != PositionUpdate_Type &&
-        i_type != StatisticsUpdate_Type &&
-        i_type != ItemChanged_Type )
-        msg_Dbg( p_intf, "New Event: type %i", i_type );
-#endif
-
     /* Actions */
     switch( i_type )
     {
@@ -174,7 +196,7 @@ void InputManager::customEvent( QEvent *event )
         break;
     case ItemChanged_Type:
         /* Ignore ItemChanged_Type event that does not apply to our input */
-        if( i_input_id == ple->i_id )
+        if( p_item == ple->p_item )
         {
             UpdateStatus();
             // UpdateName();
@@ -235,6 +257,9 @@ void InputManager::customEvent( QEvent *event )
     case ProgramChanged_Type:
         UpdateProgramEvent();
         break;
+    case EPGEvent_Type:
+        UpdateEPG();
+        break;
     default:
         msg_Warn( p_intf, "This shouldn't happen: %i", i_type );
         assert(0);
@@ -258,8 +283,9 @@ static int ItemChanged( vlc_object_t *p_this, const char *psz_var,
                         vlc_value_t oldval, vlc_value_t newval, void *param )
 {
     InputManager *im = (InputManager*)param;
+    input_item_t *p_item = static_cast<input_item_t *>(newval.p_address);
 
-    IMEvent *event = new IMEvent( ItemChanged_Type, newval.i_int );
+    IMEvent *event = new IMEvent( ItemChanged_Type, p_item );
     QApplication::postEvent( im, event );
     return VLC_SUCCESS;
 }
@@ -273,73 +299,79 @@ static int InputEvent( vlc_object_t *p_this, const char *,
     switch( newval.i_int )
     {
     case INPUT_EVENT_STATE:
-        event = new IMEvent( ItemStateChanged_Type, 0 );
+        event = new IMEvent( ItemStateChanged_Type );
         break;
     case INPUT_EVENT_RATE:
-        event = new IMEvent( ItemRateChanged_Type, 0 );
+        event = new IMEvent( ItemRateChanged_Type );
         break;
-    case INPUT_EVENT_TIMES:
-        event = new IMEvent( PositionUpdate_Type, 0 );
+    case INPUT_EVENT_POSITION:
+    //case INPUT_EVENT_LENGTH:
+        event = new IMEvent( PositionUpdate_Type );
         break;
 
     case INPUT_EVENT_TITLE:
     case INPUT_EVENT_CHAPTER:
-        event = new IMEvent( ItemTitleChanged_Type, 0 );
+        event = new IMEvent( ItemTitleChanged_Type );
         break;
 
     case INPUT_EVENT_ES:
-        event = new IMEvent( ItemEsChanged_Type, 0 );
+        event = new IMEvent( ItemEsChanged_Type );
         break;
     case INPUT_EVENT_TELETEXT:
-        event = new IMEvent( ItemTeletextChanged_Type, 0 );
+        event = new IMEvent( ItemTeletextChanged_Type );
         break;
 
     case INPUT_EVENT_STATISTICS:
-        event = new IMEvent( StatisticsUpdate_Type, 0 );
+        event = new IMEvent( StatisticsUpdate_Type );
         break;
 
     case INPUT_EVENT_VOUT:
-        event = new IMEvent( InterfaceVoutUpdate_Type, 0 );
+        event = new IMEvent( InterfaceVoutUpdate_Type );
         break;
     case INPUT_EVENT_AOUT:
-        event = new IMEvent( InterfaceAoutUpdate_Type, 0 );
+        event = new IMEvent( InterfaceAoutUpdate_Type );
         break;
 
     case INPUT_EVENT_ITEM_META: /* Codec MetaData + Art */
-        event = new IMEvent( MetaChanged_Type, 0 );
+        event = new IMEvent( MetaChanged_Type );
         break;
     case INPUT_EVENT_ITEM_INFO: /* Codec Info */
-        event = new IMEvent( InfoChanged_Type, 0 );
+        event = new IMEvent( InfoChanged_Type );
         break;
     case INPUT_EVENT_ITEM_NAME:
-        event = new IMEvent( NameChanged_Type, 0 );
+        event = new IMEvent( NameChanged_Type );
         break;
 
     case INPUT_EVENT_AUDIO_DELAY:
     case INPUT_EVENT_SUBTITLE_DELAY:
-        event = new IMEvent( SynchroChanged_Type, 0 );
+        event = new IMEvent( SynchroChanged_Type );
         break;
 
     case INPUT_EVENT_CACHE:
-        event = new IMEvent( CachingEvent_Type, 0 );
+        event = new IMEvent( CachingEvent_Type );
         break;
 
     case INPUT_EVENT_BOOKMARK:
-        event = new IMEvent( BookmarksChanged_Type, 0 );
+        event = new IMEvent( BookmarksChanged_Type );
         break;
 
     case INPUT_EVENT_RECORD:
-        event = new IMEvent( RecordingEvent_Type, 0 );
+        event = new IMEvent( RecordingEvent_Type );
         break;
 
     case INPUT_EVENT_PROGRAM:
         /* This is for PID changes */
-        event = new IMEvent( ProgramChanged_Type, 0 );
+        event = new IMEvent( ProgramChanged_Type );
+        break;
+
+    case INPUT_EVENT_ITEM_EPG:
+        /* EPG data changed */
+        event = new IMEvent( EPGEvent_Type );
         break;
 
     case INPUT_EVENT_SIGNAL:
         /* This is for capture-card signals */
-        /* event = new IMEvent( SignalChanged_Type, 0 );
+        /* event = new IMEvent( SignalChanged_Type );
         break; */
     default:
         event = NULL;
@@ -355,7 +387,7 @@ static int VbiEvent( vlc_object_t *, const char *,
                      vlc_value_t, vlc_value_t, void *param )
 {
     InputManager *im = (InputManager*)param;
-    IMEvent *event = new IMEvent( ItemTeletextChanged_Type, 0 );
+    IMEvent *event = new IMEvent( ItemTeletextChanged_Type );
 
     QApplication::postEvent( im, event );
     return VLC_SUCCESS;
@@ -364,10 +396,11 @@ static int VbiEvent( vlc_object_t *, const char *,
 void InputManager::UpdatePosition()
 {
     /* Update position */
-    int i_length, i_time; /* Int is enough, since we store seconds */
+    int i_length;
+    int64_t i_time;
     float f_pos;
     i_length = var_GetTime(  p_input , "length" ) / 1000000;
-    i_time = var_GetTime(  p_input , "time") / 1000000;
+    i_time = var_GetTime(  p_input , "time");
     f_pos = var_GetFloat(  p_input , "position" );
     emit positionUpdated( f_pos, i_time, i_length );
 }
@@ -376,6 +409,7 @@ void InputManager::UpdateNavigation()
 {
     /* Update navigation status */
     vlc_value_t val; val.i_int = 0;
+    vlc_value_t val2; val2.i_int = 0;
 
     if( hasInput() )
         var_Change( p_input, "title", VLC_VAR_CHOICESCOUNT, &val, NULL );
@@ -385,10 +419,9 @@ void InputManager::UpdateNavigation()
         emit titleChanged( true );
         msg_Dbg( p_intf, "Title %i", val.i_int );
         /* p_input != NULL since val.i_int != 0 */
-        val.i_int = 0;
-        var_Change( p_input, "chapter", VLC_VAR_CHOICESCOUNT, &val, NULL );
-        emit chapterChanged( (val.i_int > 0) );
-        msg_Dbg( p_intf, "Chapter: %i", val.i_int );
+        var_Change( p_input, "chapter", VLC_VAR_CHOICESCOUNT, &val2, NULL );
+        emit chapterChanged( (val2.i_int > 1) || ( val2.i_int > 0 && val.i_int > 1 ) );
+        msg_Dbg( p_intf, "Chapter: %i", val2.i_int );
     }
     else
         emit titleChanged( false );
@@ -397,24 +430,23 @@ void InputManager::UpdateNavigation()
 void InputManager::UpdateStatus()
 {
     /* Update playing status */
-    vlc_value_t val; val.i_int = 0;
-    var_Get( p_input, "state", &val );
-    if( i_old_playing_status != val.i_int )
+    int state = var_GetInteger( p_input, "state" );
+    if( i_old_playing_status != state )
     {
-        i_old_playing_status = val.i_int;
-        emit statusChanged( val.i_int );
+        i_old_playing_status = state;
+        emit statusChanged( state );
     }
 }
 
 void InputManager::UpdateRate()
 {
     /* Update Rate */
-    int i_new_rate = var_GetInteger( p_input, "rate");
-    if( i_new_rate != i_rate )
+    float f_new_rate = var_GetFloat( p_input, "rate" );
+    if( f_new_rate != f_rate )
     {
-        i_rate = i_new_rate;
+        f_rate = f_new_rate;
         /* Update rate */
-        emit rateChanged( i_rate );
+        emit rateChanged( f_rate );
     }
 }
 
@@ -492,21 +524,23 @@ void InputManager::UpdateTeletext()
             int i_page = 100;
             bool b_transparent = false;
 
-            vlc_object_t *p_vbi = (vlc_object_t *)
-                vlc_object_find_name( p_input, "zvbi", FIND_CHILD );
+            if( p_input_vbi )
+            {
+                var_DelCallback( p_input_vbi, "vbi-page", VbiEvent, this );
+                vlc_object_release( p_input_vbi );
+            }
 
-            if( p_vbi )
+            if( input_GetEsObjects( p_input, i_teletext_es, &p_input_vbi, NULL, NULL ) )
+                p_input_vbi = NULL;
+
+            if( p_input_vbi )
             {
-                /* We deleted it (if not here, it does not harm), because
-                 * var_AddCallback will silently add a duplicated one */
-                var_DelCallback( p_vbi, "vbi-page", VbiEvent, this );
                 /* This callback is not remove explicitly, but interfaces
                  * are guaranted to outlive input */
-                var_AddCallback( p_vbi, "vbi-page", VbiEvent, this );
+                var_AddCallback( p_input_vbi, "vbi-page", VbiEvent, this );
 
-                i_page = var_GetInteger( p_vbi, "vbi-page" );
-                b_transparent = !var_GetBool( p_vbi, "vbi-opaque" );
-                vlc_object_release( p_vbi );
+                i_page = var_GetInteger( p_input_vbi, "vbi-page" );
+                b_transparent = !var_GetBool( p_input_vbi, "vbi-opaque" );
             }
             emit newTelexPageSet( i_page );
             emit teletextTransparencyActivated( b_transparent );
@@ -521,6 +555,14 @@ void InputManager::UpdateTeletext()
     }
 }
 
+void InputManager::UpdateEPG()
+{
+    if( hasInput() )
+    {
+       emit epgChanged();
+    }
+}
+
 void InputManager::UpdateVout()
 {
     if( hasInput() )
@@ -564,7 +606,7 @@ void InputManager::UpdateCaching()
     if( f_newCache != f_cache )
     {
         f_cache = f_newCache;
-        /* Update rate */
+        /* Update cache */
         emit cachingChanged( f_cache );
     }
 }
@@ -573,9 +615,7 @@ void InputManager::requestArtUpdate()
 {
     if( hasInput() )
     {
-        playlist_t *p_playlist = pl_Hold( p_intf );
-        playlist_AskForArtEnqueue( p_playlist, input_GetItem( p_input ), pl_Unlocked );
-        pl_Release( p_intf );
+        playlist_AskForArtEnqueue( pl_Get(p_intf), input_GetItem( p_input ) );
     }
     else
     {
@@ -585,25 +625,39 @@ void InputManager::requestArtUpdate()
     }
 }
 
+const QString InputManager::decodeArtURL( input_item_t *p_item )
+{
+    assert( p_item );
+
+    char *psz_art = input_item_GetArtURL( p_item );
+    if( psz_art )
+    {
+        char *psz = make_path( psz_art );
+        free( psz_art );
+        psz_art = psz;
+    }
+
+#if 0
+    /* Taglib seems to define a attachment://, It won't work yet */
+    url = url.replace( "attachment://", "" );
+#endif
+    return qfu( psz_art ? psz_art : "" );
+}
+
 void InputManager::UpdateArt()
 {
     QString url;
 
     if( hasInput() )
-    {
-        char *psz_art = input_item_GetArtURL( input_GetItem( p_input ) );
-        if( psz_art && !strncmp( psz_art, "file://", 7 ) &&
-                decode_URI( psz_art + 7 ) )
-#ifdef WIN32
-            url = qfu( psz_art + 8 ); // Remove extra / starting on Win32.
-#else
-            url = qfu( psz_art + 7 );
-#endif
-        free( psz_art );
-    }
+        url = decodeArtURL( input_GetItem( p_input ) );
+
+    /* the art hasn't changed, no need to update */
+    if(artUrl == url)
+        return;
 
     /* Update Art meta */
-    emit artChanged( url );
+    artUrl = url;
+    emit artChanged( artUrl );
 }
 
 inline void InputManager::UpdateStats()
@@ -611,14 +665,14 @@ inline void InputManager::UpdateStats()
     emit statisticsUpdated( input_GetItem( p_input ) );
 }
 
-inline void InputManager::UpdateMeta( int id )
+inline void InputManager::UpdateMeta( input_item_t *p_item )
 {
-    emit metaChanged( id );
+    emit metaChanged( p_item );
 }
 
 inline void InputManager::UpdateMeta()
 {
-    emit metaChanged( input_GetItem( p_input ) );
+    emit currentMetaChanged( input_GetItem( p_input ) );
 }
 
 inline void InputManager::UpdateInfo()
@@ -648,6 +702,7 @@ void InputManager::sliderUpdate( float new_pos )
 {
     if( hasInput() )
         var_SetFloat( p_input, "position", new_pos );
+    emit seekRequested( new_pos );
 }
 
 /* User togglePlayPause */
@@ -658,7 +713,6 @@ void InputManager::togglePlayPause()
         int state = var_GetInteger( p_input, "state" );
         state = ( state != PLAYING_S ) ? PLAYING_S : PAUSE_S;
         var_SetInteger( p_input, "state", state );
-        emit statusChanged( state );
     }
 }
 
@@ -667,8 +721,8 @@ void InputManager::sectionPrev()
     if( hasInput() )
     {
         int i_type = var_Type( p_input, "next-chapter" );
-        var_SetVoid( p_input, (i_type & VLC_VAR_TYPE) != 0 ?
-                            "prev-chapter":"prev-title" );
+        var_TriggerCallback( p_input, (i_type & VLC_VAR_TYPE) != 0 ?
+                             "prev-chapter":"prev-title" );
     }
 }
 
@@ -677,8 +731,8 @@ void InputManager::sectionNext()
     if( hasInput() )
     {
         int i_type = var_Type( p_input, "next-chapter" );
-        var_SetVoid( p_input, (i_type & VLC_VAR_TYPE) != 0 ?
-                            "next-chapter":"next-title" );
+        var_TriggerCallback( p_input, (i_type & VLC_VAR_TYPE) != 0 ?
+                             "next-chapter":"next-title" );
     }
 }
 
@@ -698,7 +752,7 @@ void InputManager::sectionMenu()
             if( !strcmp( text.p_list->p_values[i].psz_string, "Title" ) )
                 root = i;
         }
-        var_Change( p_input, "title  0", VLC_VAR_FREELIST, &val, &text );
+        var_FreeList( &val, &text );
 
         var_SetInteger( p_input, "title  0", root );
     }
@@ -711,20 +765,14 @@ void InputManager::sectionMenu()
 /* Set a new Teletext Page */
 void InputManager::telexSetPage( int page )
 {
-    if( hasInput() )
+    if( hasInput() && p_input_vbi )
     {
         const int i_teletext_es = var_GetInteger( p_input, "teletext-es" );
 
         if( i_teletext_es >= 0 )
         {
-            vlc_object_t *p_vbi = (vlc_object_t *) vlc_object_find_name( p_input,
-                        "zvbi", FIND_CHILD );
-            if( p_vbi )
-            {
-                var_SetInteger( p_vbi, "vbi-page", page );
-                vlc_object_release( p_vbi );
-                emit newTelexPageSet( page );
-            }
+            var_SetInteger( p_input_vbi, "vbi-page", page );
+            emit newTelexPageSet( page );
         }
     }
 }
@@ -732,16 +780,10 @@ void InputManager::telexSetPage( int page )
 /* Set the transparency on teletext */
 void InputManager::telexSetTransparency( bool b_transparentTelextext )
 {
-    if( hasInput() )
+    if( hasInput() && p_input_vbi )
     {
-        vlc_object_t *p_vbi = (vlc_object_t *) vlc_object_find_name( p_input,
-                    "zvbi", FIND_CHILD );
-        if( p_vbi )
-        {
-            var_SetBool( p_vbi, "vbi-opaque", !b_transparentTelextext );
-            vlc_object_release( p_vbi );
-            emit teletextTransparencyActivated( b_transparentTelextext );
-        }
+        var_SetBool( p_input_vbi, "vbi-opaque", !b_transparentTelextext );
+        emit teletextTransparencyActivated( b_transparentTelextext );
     }
 }
 
@@ -766,7 +808,7 @@ void InputManager::activateTeletext( bool b_enable )
                 i = 0;
             var_SetInteger( p_input, "spu-es", b_enable ? list.p_list->p_values[i].i_int : -1 );
         }
-        var_Change( p_input, "teletext-es", VLC_VAR_FREELIST, &list, &text );
+        var_FreeList( &list, &text );
     }
 }
 
@@ -774,21 +816,21 @@ void InputManager::reverse()
 {
     if( hasInput() )
     {
-        int i_rate = var_GetInteger( p_input, "rate" );
-        var_SetInteger( p_input, "rate", -i_rate );
+        float f_rate = var_GetFloat( p_input, "rate" );
+        var_SetFloat( p_input, "rate", -f_rate );
     }
 }
 
 void InputManager::slower()
 {
     if( hasInput() )
-        var_SetVoid( p_input, "rate-slower" );
+        var_TriggerCallback( p_input, "rate-slower" );
 }
 
 void InputManager::faster()
 {
     if( hasInput() )
-        var_SetVoid( p_input, "rate-faster" );
+        var_TriggerCallback( p_input, "rate-faster" );
 }
 
 void InputManager::littlefaster()
@@ -804,18 +846,19 @@ void InputManager::littleslower()
 void InputManager::normalRate()
 {
     if( hasInput() )
-        var_SetInteger( p_input, "rate", INPUT_RATE_DEFAULT );
+        var_SetFloat( p_input, "rate", 1. );
 }
 
 void InputManager::setRate( int new_rate )
 {
     if( hasInput() )
-        var_SetInteger( p_input, "rate", new_rate );
+        var_SetFloat( p_input, "rate",
+                      (float)INPUT_RATE_DEFAULT / (float)new_rate );
 }
 
 void InputManager::jumpFwd()
 {
-    int i_interval = config_GetInt( p_input, "short-jump-size" );
+    int i_interval = var_InheritInteger( p_input, "short-jump-size" );
     if( i_interval > 0 )
     {
         mtime_t val = (mtime_t)(i_interval) * 1000000L;
@@ -825,7 +868,7 @@ void InputManager::jumpFwd()
 
 void InputManager::jumpBwd()
 {
-    int i_interval = config_GetInt( p_input, "short-jump-size" );
+    int i_interval = var_InheritInteger( p_input, "short-jump-size" );
     if( i_interval > 0 )
     {
         mtime_t val = -1 *(mtime_t)(i_interval) * 1000000L;
@@ -843,26 +886,25 @@ void InputManager::setAtoB()
     {
         timeB = var_GetTime( THEMIM->getInput(), "time"  );
         var_SetTime( THEMIM->getInput(), "time" , timeA );
-        CONNECT( this, positionUpdated( float, int, int ),
-                 this, AtoBLoop( float, int, int ) );
+        CONNECT( this, positionUpdated( float, int64_t, int ),
+                 this, AtoBLoop( float, int64_t, int ) );
     }
     else
     {
         timeA = 0;
         timeB = 0;
-        disconnect( this, SIGNAL( positionUpdated( float, int, int ) ),
-                    this, SLOT( AtoBLoop( float, int, int ) ) );
+        disconnect( this, SIGNAL( positionUpdated( float, int64_t, int ) ),
+                    this, SLOT( AtoBLoop( float, int64_t, int ) ) );
     }
     emit AtoBchanged( (timeA != 0 ), (timeB != 0 ) );
 }
 
 /* Function called regularly when in an AtoB loop */
-void InputManager::AtoBLoop( float, int i_time, int )
+void InputManager::AtoBLoop( float, int64_t i_time, int )
 {
     if( timeB )
     {
-        if( ( i_time >= (int)( timeB/1000000 ) )
-            || ( i_time < (int)( timeA/1000000 ) ) )
+        if( i_time >= timeB || i_time < timeA )
             var_SetTime( THEMIM->getInput(), "time" , timeA );
     }
 }
@@ -883,18 +925,33 @@ MainInputManager::MainInputManager( intf_thread_t *_p_intf )
     var_AddCallback( THEPL, "item-change", ItemChanged, im );
     var_AddCallback( THEPL, "item-current", PLItemChanged, this );
     var_AddCallback( THEPL, "activity", PLItemChanged, this );
+    var_AddCallback( THEPL, "leaf-to-parent", LeafToParent, this );
+    var_AddCallback( THEPL, "playlist-item-append", PLItemAppended, this );
+    var_AddCallback( THEPL, "playlist-item-deleted", PLItemRemoved, this );
+    var_AddCallback( THEPL, "random", RandomChanged, this );
+    var_AddCallback( THEPL, "repeat", RepeatChanged, this );
+    var_AddCallback( THEPL, "loop", LoopChanged, this );
 
-    var_AddCallback( p_intf->p_libvlc, "volume-change", VolumeChanged, this );
+    var_AddCallback( THEPL, "volume-change", VolumeChanged, this );
+    var_AddCallback( THEPL, "volume-muted", SoundMuteChanged, this );
 
     /* Warn our embedded IM about input changes */
-    CONNECT( this, inputChanged( input_thread_t * ),
-             im, setInput( input_thread_t * ) );
+    DCONNECT( this, inputChanged( input_thread_t * ),
+              im, setInput( input_thread_t * ) );
 
     /* emit check if playlist has already started playing */
-    IMEvent *event = new IMEvent( ItemChanged_Type,
-                                  var_GetInteger( THEPL, "item-current" ) );
-    customEvent( event );
-    delete event;
+    input_thread_t *p_input = playlist_CurrentInput( THEPL );
+    if( p_input )
+    {
+        input_item_t *p_item = input_GetItem( p_input );
+        if( p_item )
+        {
+            IMEvent *event = new IMEvent( ItemChanged_Type, p_item );
+            customEvent( event );
+            delete event;
+        }
+        vlc_object_release( p_input );
+    }
 }
 
 MainInputManager::~MainInputManager()
@@ -906,12 +963,20 @@ MainInputManager::~MainInputManager()
        vlc_object_release( p_input );
     }
 
-    var_DelCallback( p_intf->p_libvlc, "volume-change", VolumeChanged, this );
+    var_DelCallback( THEPL, "volume-change", VolumeChanged, this );
+    var_DelCallback( THEPL, "volume-muted", SoundMuteChanged, this );
 
     var_DelCallback( THEPL, "activity", PLItemChanged, this );
     var_DelCallback( THEPL, "item-change", ItemChanged, im );
+    var_DelCallback( THEPL, "leaf-to-parent", LeafToParent, this );
 
     var_DelCallback( THEPL, "item-current", PLItemChanged, this );
+    var_DelCallback( THEPL, "playlist-item-append", PLItemAppended, this );
+    var_DelCallback( THEPL, "playlist-item-deleted", PLItemRemoved, this );
+    var_DelCallback( THEPL, "random", RandomChanged, this );
+    var_DelCallback( THEPL, "repeat", RepeatChanged, this );
+    var_DelCallback( THEPL, "loop", LoopChanged, this );
+
 }
 
 vout_thread_t* MainInputManager::getVout()
@@ -927,14 +992,39 @@ aout_instance_t * MainInputManager::getAout()
 void MainInputManager::customEvent( QEvent *event )
 {
     int type = event->type();
-    if ( type != ItemChanged_Type && type != VolumeChanged_Type )
-        return;
+
+    PLEvent *plEv;
+    IMEvent *imEv;
 
     // msg_Dbg( p_intf, "New MainIM Event of type: %i", type );
-    if( type == VolumeChanged_Type )
+    switch( type )
     {
+    case VolumeChanged_Type:
         emit volumeChanged();
         return;
+    case SoundMuteChanged_Type:
+        emit soundMuteChanged();
+        return;
+    case PLItemAppended_Type:
+        plEv = static_cast<PLEvent*>( event );
+        emit playlistItemAppended( plEv->i_item, plEv->i_parent );
+        return;
+    case PLItemRemoved_Type:
+        plEv = static_cast<PLEvent*>( event );
+        emit playlistItemRemoved( plEv->i_item );
+        return;
+    case RandomChanged_Type:
+        emit randomChanged( var_GetBool( THEPL, "random" ) );
+        return;
+    case LoopChanged_Type:
+    case RepeatChanged_Type:
+        notifyRepeatLoop();
+        return;
+    case LeafToParent_Type:
+        imEv = static_cast<IMEvent*>( event );
+        emit leafBecameParent( imEv->p_item );
+    default:
+        if( type != ItemChanged_Type ) return;
     }
 
     /* Should be PLItemChanged Event */
@@ -968,13 +1058,11 @@ void MainInputManager::customEvent( QEvent *event )
             p_input = NULL;
         }
         /* we are working as a dialogs provider */
-        playlist_t *p_playlist = pl_Hold( p_intf );
-        p_input = playlist_CurrentInput( p_playlist );
+        p_input = playlist_CurrentInput( pl_Get(p_intf) );
         if( p_input )
         {
             emit inputChanged( p_input );
         }
-        pl_Release( p_intf );
     }
 }
 
@@ -1003,18 +1091,81 @@ void MainInputManager::togglePlayPause()
         getIM()->togglePlayPause();
 }
 
+void MainInputManager::play()
+{
+    /* No input, play */
+    if( !p_input )
+        playlist_Play( THEPL );
+    else
+    {
+        if( PLAYING_S != var_GetInteger( p_input, "state" ) )
+        {
+            getIM()->togglePlayPause();
+        }
+    }
+}
+
+void MainInputManager::pause()
+{
+    if(p_input && PLAYING_S == var_GetInteger( p_input, "state" ) )
+    {
+        getIM()->togglePlayPause();
+    }
+}
+
+void MainInputManager::toggleRandom()
+{
+    var_ToggleBool( THEPL, "random" );
+}
+
+void MainInputManager::notifyRepeatLoop()
+{
+    int i_value = var_GetBool( THEPL, "loop" ) * REPEAT_ALL
+              + var_GetBool( THEPL, "repeat" ) * REPEAT_ONE;
+
+    emit repeatLoopChanged( i_value );
+}
+
+void MainInputManager::loopRepeatLoopStatus()
+{
+    /* Toggle Normal -> Loop -> Repeat -> Normal ... */
+    if( var_GetBool( THEPL, "repeat" ) )
+        var_SetBool( THEPL, "repeat", false );
+    else if( var_GetBool( THEPL, "loop" ) )
+    {
+        var_SetBool( THEPL, "loop", false );
+        var_SetBool( THEPL, "repeat", true );
+    }
+    else
+        var_SetBool( THEPL, "loop", true );
+}
+
 void MainInputManager::activatePlayQuit( bool b_exit )
 {
     var_SetBool( THEPL, "play-and-exit", b_exit );
 }
 
-/* Static callbacks for MIM */
+
+/****************************
+ * Static callbacks for MIM *
+ ****************************/
 static int PLItemChanged( vlc_object_t *p_this, const char *psz_var,
+                        vlc_value_t oldval, vlc_value_t, void *param )
+{
+    MainInputManager *mim = (MainInputManager*)param;
+
+    IMEvent *event = new IMEvent( ItemChanged_Type );
+    QApplication::postEvent( mim, event );
+    return VLC_SUCCESS;
+}
+
+static int LeafToParent( vlc_object_t *p_this, const char *psz_var,
                         vlc_value_t oldval, vlc_value_t newval, void *param )
 {
     MainInputManager *mim = (MainInputManager*)param;
 
-    IMEvent *event = new IMEvent( ItemChanged_Type, newval.i_int );
+    IMEvent *event = new IMEvent( LeafToParent_Type,
+                                  static_cast<input_item_t*>( newval.p_address ) );
     QApplication::postEvent( mim, event );
     return VLC_SUCCESS;
 }
@@ -1024,8 +1175,68 @@ static int VolumeChanged( vlc_object_t *p_this, const char *psz_var,
 {
     MainInputManager *mim = (MainInputManager*)param;
 
-    IMEvent *event = new IMEvent( VolumeChanged_Type, newval.i_int );
+    IMEvent *event = new IMEvent( VolumeChanged_Type );
     QApplication::postEvent( mim, event );
     return VLC_SUCCESS;
 }
 
+static int SoundMuteChanged( vlc_object_t *p_this, const char *psz_var,
+                        vlc_value_t oldval, vlc_value_t newval, void *param )
+{
+    MainInputManager *mim = (MainInputManager*)param;
+
+    IMEvent *event = new IMEvent( SoundMuteChanged_Type );
+    QApplication::postEvent( mim, event );
+    return VLC_SUCCESS;
+}
+
+static int PLItemAppended
+( vlc_object_t * obj, const char *var, vlc_value_t old, vlc_value_t cur, void *data )
+{
+    MainInputManager *mim = static_cast<MainInputManager*>(data);
+    playlist_add_t *p_add = static_cast<playlist_add_t*>( cur.p_address );
+
+    PLEvent *event = new PLEvent( PLItemAppended_Type, p_add->i_item, p_add->i_node  );
+    QApplication::postEvent( mim, event );
+    return VLC_SUCCESS;
+}
+static int PLItemRemoved
+( vlc_object_t * obj, const char *var, vlc_value_t old, vlc_value_t cur, void *data )
+{
+    MainInputManager *mim = static_cast<MainInputManager*>(data);
+
+    PLEvent *event = new PLEvent( PLItemRemoved_Type, cur.i_int, 0  );
+    QApplication::postEvent( mim, event );
+    return VLC_SUCCESS;
+}
+
+static int RandomChanged
+( vlc_object_t * obj, const char *var, vlc_value_t old, vlc_value_t cur, void *data )
+{
+    MainInputManager *mim = static_cast<MainInputManager*>(data);
+
+    IMEvent *event = new IMEvent( RandomChanged_Type );
+    QApplication::postEvent( mim, event );
+    return VLC_SUCCESS;
+}
+
+/* Probably could be merged with next callback */
+static int LoopChanged
+( vlc_object_t * obj, const char *var, vlc_value_t old, vlc_value_t cur, void *data )
+{
+    MainInputManager *mim = static_cast<MainInputManager*>(data);
+
+    IMEvent *event = new IMEvent( LoopChanged_Type );
+    QApplication::postEvent( mim, event );
+    return VLC_SUCCESS;
+}
+
+static int RepeatChanged
+( vlc_object_t * obj, const char *var, vlc_value_t old, vlc_value_t cur, void *data )
+{
+    MainInputManager *mim = static_cast<MainInputManager*>(data);
+
+    IMEvent *event = new IMEvent( RepeatChanged_Type );
+    QApplication::postEvent( mim, event );
+    return VLC_SUCCESS;
+}
