@@ -2,7 +2,7 @@
  * ctrl_tree.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: 50b46750269edb20e16d5690d87f073f8a3fe3dd $
+ * $Id: 27ffdd29d61c719171541247aada8157e1864b0c $
  *
  * Authors: Antoine Cellerier <dionoea@videolan.org>
  *          Clément Stenac <zorglub@videolan.org>
@@ -35,7 +35,7 @@
 #include "../events/evt_key.hpp"
 #include "../events/evt_mouse.hpp"
 #include "../events/evt_scroll.hpp"
-#include "vlc_keys.h"
+#include <vlc_keys.h>
 
 #define SCROLL_STEP 0.05
 #define LINE_INTERVAL 1  // Number of pixels inserted between 2 lines
@@ -138,6 +138,8 @@ int CtrlTree::maxItems()
 void CtrlTree::onUpdate( Subject<VarTree, tree_update> &rTree,
                          tree_update *arg )
 {
+    m_firstPos = m_flat ? m_rTree.firstLeaf() : m_rTree.begin();
+
     if( arg->i_type == 0 ) // Item update
     {
         if( arg->b_active_item )
@@ -150,7 +152,6 @@ void CtrlTree::onUpdate( Subject<VarTree, tree_update> &rTree,
     /// \todo handle delete in a more clever way
     else if ( arg->i_type == 1 ) // Global change or deletion
     {
-        m_firstPos = m_flat ? m_rTree.firstLeaf() : m_rTree.begin();
         makeImage();
     }
     else if ( arg->i_type == 2 ) // Item-append
@@ -244,13 +245,11 @@ void CtrlTree::onResize()
     // Redraw the control if the position has changed
     m_firstPos = it;
     makeImage();
-    notifyLayout();
 }
 
 void CtrlTree::onPositionChange()
 {
     makeImage();
-    notifyLayout();
 }
 
 void CtrlTree::handleEvent( EvtGeneric &rEvent )
@@ -331,6 +330,14 @@ void CtrlTree::handleEvent( EvtGeneric &rEvent )
             notifyLayout();
             return;
         }
+        else
+        {
+            // other keys to be forwarded to vlc core
+            EvtKey& rEvtKey = (EvtKey&)rEvent;
+            var_SetInteger( getIntf()->p_libvlc, "key-pressed",
+                            rEvtKey.getModKey() );
+            return;
+        }
 
 
         for( it = m_flat ? m_rTree.firstLeaf() : m_rTree.begin();
@@ -430,7 +437,7 @@ void CtrlTree::handleEvent( EvtGeneric &rEvent )
                     }
                 }
             }
-            else if( key == KEY_ENTER || key == KEY_SPACE )
+            else if( key == KEY_ENTER || key == ' ' )
             {
                 // Go up one level (and close node)
                 if( &*it == m_pLastSelected )
@@ -574,7 +581,10 @@ void CtrlTree::handleEvent( EvtGeneric &rEvent )
 
     else if( rEvent.getAsString().find( "scroll" ) != string::npos )
     {
-        int direction = ((EvtScroll&)rEvent).getDirection();
+        // XXX ctrl_slider.cpp has two more (but slightly different)
+        // XXX implementations of `scroll'. Figure out where it belongs.
+
+        int direction = static_cast<EvtScroll&>(rEvent).getDirection();
 
         double percentage = m_rTree.getPositionVar().get();
         double step = 2.0 / (double)( m_flat ? m_rTree.countLeafs()
@@ -622,17 +632,14 @@ void CtrlTree::handleEvent( EvtGeneric &rEvent )
 bool CtrlTree::mouseOver( int x, int y ) const
 {
     const Position *pPos = getPosition();
-    return ( pPos
-       ? x >= 0 && x <= pPos->getWidth() && y >= 0 && y <= pPos->getHeight()
-       : false);
+    return !pPos ? false :
+        x >= 0 && x <= pPos->getWidth() && y >= 0 && y <= pPos->getHeight();
 }
 
 void CtrlTree::draw( OSGraphics &rImage, int xDest, int yDest )
 {
     if( m_pImage )
-    {
         rImage.drawGraphics( *m_pImage, 0, 0, xDest, yDest );
-    }
 }
 
 bool CtrlTree::ensureVisible( VarTree::Iterator item )
@@ -776,7 +783,9 @@ void CtrlTree::makeImage()
         for( int yPos = 0; yPos < height; yPos += i_itemHeight )
         {
             int rectHeight = __MIN( i_itemHeight, height - yPos );
-            if( it != m_rTree.end() )
+            if( it == m_rTree.end() )
+                m_pImage->fillRect( 0, yPos, width, rectHeight, bgColor );
+            else
             {
                 uint32_t color = ( it->m_selected ? m_selColor : bgColor );
                 m_pImage->fillRect( 0, yPos, width, rectHeight, color );
@@ -786,10 +795,6 @@ void CtrlTree::makeImage()
                                 : m_rTree.getNextVisibleItem( it );
                 } while( it != m_rTree.end() && it->m_deleted );
             }
-            else
-            {
-                m_pImage->fillRect( 0, yPos, width, rectHeight, bgColor );
-            }
             bgColor = ( bgColor == m_bgColor1 ? m_bgColor2 : m_bgColor1 );
         }
     }
