@@ -2,10 +2,10 @@
  * var_manager.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: var_manager.cpp 11973 2005-08-02 23:26:22Z asmax $
+ * $Id: 053ac95026414f99f7b215e47798e26f13723e78 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include "var_manager.hpp"
@@ -48,6 +48,7 @@ VarManager::~VarManager()
         m_anonVarList.pop_back();
     }
 
+
     delete m_pTooltipText;
 
     // Warning! the help text must be the last variable to be deleted,
@@ -73,11 +74,8 @@ VarManager *VarManager::instance( intf_thread_t *pIntf )
 
 void VarManager::destroy( intf_thread_t *pIntf )
 {
-    if( pIntf->p_sys->p_varManager )
-    {
-        delete pIntf->p_sys->p_varManager;
-        pIntf->p_sys->p_varManager = NULL;
-    }
+    delete pIntf->p_sys->p_varManager;
+    pIntf->p_sys->p_varManager = NULL;
 }
 
 
@@ -85,6 +83,8 @@ void VarManager::registerVar( const VariablePtr &rcVar, const string &rName )
 {
     m_varMap[rName] = rcVar;
     m_varList.push_front( rName );
+
+    m_anonVarList.push_back( rcVar );
 }
 
 
@@ -115,7 +115,7 @@ Variable *VarManager::getVar( const string &rName, const string &rType )
         // Check the variable type
         if( pVar->getType() != rType )
         {
-            msg_Warn( getIntf(), "Variable %s has incorrect type (%s instead"
+            msg_Warn( getIntf(), "variable %s has incorrect type (%s instead"
                       " of (%s)", rName.c_str(), pVar->getType().c_str(),
                       rType.c_str() );
             return NULL;
@@ -131,3 +131,15 @@ Variable *VarManager::getVar( const string &rName, const string &rType )
     }
 }
 
+
+void VarManager::registerConst( const string &rName, const string &rValue)
+{
+    m_constMap[rName] = rValue;
+}
+
+
+string VarManager::getConst( const string &rName )
+{
+    return m_constMap[rName];
+}
+
