@@ -2,7 +2,7 @@
  * mpjpeg.c: mime multipart jpeg  muxer module for vlc
  *****************************************************************************
  * Copyright (C) 2001, 2002, 2006 the VideoLAN team
- * $Id: 1511373046365afbf98d7e8e9c8a82055706cc28 $
+ * $Id: 1de2ec4cbe4774a494931613af5b5a17c00da41c $
  *
  * Authors: Sigmund Augdal Helberg <dnumgis@videolan.org>
  *
@@ -131,16 +131,8 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
     }
 
     msg_Dbg( p_mux, "adding input" );
-    if( p_input->p_fmt->i_codec != VLC_FOURCC('M','J','P','G') &&
-        p_input->p_fmt->i_codec != VLC_FOURCC('m','j','p','g') &&
-        p_input->p_fmt->i_codec != VLC_FOURCC('j','p','e','g') &&
-        p_input->p_fmt->i_codec != VLC_FOURCC('J','P','E','G') &&
-        p_input->p_fmt->i_codec != VLC_FOURCC('J','F','I','F') &&
-        p_input->p_fmt->i_codec != VLC_FOURCC('J','P','G','L') &&
-        p_input->p_fmt->i_codec != VLC_FOURCC('m','j','p','a') )
-    {
+    if( p_input->p_fmt->i_codec != VLC_CODEC_MJPG )
         return VLC_EGENERIC;
-    }
 
     return VLC_SUCCESS;
 }
