@@ -2,7 +2,7 @@
  * es_out_timeshift.c: Es Out timeshift.
  *****************************************************************************
  * Copyright (C) 2008 Laurent Aimar
- * $Id: 19f4d211056c8e09594239a8c28a7195f156f082 $
+ * $Id: 4aa997feef1b801c42af8cc9c721988e8e87fccd $
  *
  * Authors: Laurent Aimar < fenrir _AT_ videolan _DOT_ org>
  *
@@ -30,7 +30,6 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <errno.h>
 #include <assert.h>
 #if defined (WIN32) && !defined (UNDER_CE)
 #  include <direct.h>
@@ -40,8 +39,10 @@
 #endif
 
 #include <vlc_common.h>
-#include <vlc_charset.h>
-
+#include <vlc_fs.h>
+#ifdef WIN32
+#  include <vlc_charset.h>
+#endif
 #include <vlc_input.h>
 #include <vlc_es_out.h>
 #include <vlc_block.h>
@@ -131,9 +132,10 @@ typedef struct attribute_packed
         struct
         {
             mtime_t i_pts_delay;
+            mtime_t i_pts_jitter;
             int     i_cr_average;
         } jitter;
-    };
+    } u;
 } ts_cmd_control_t;
 
 typedef struct attribute_packed
@@ -146,7 +148,7 @@ typedef struct attribute_packed
         ts_cmd_del_t     del;
         ts_cmd_send_t    send;
         ts_cmd_control_t control;
-    };
+    } u;
 } ts_cmd_t;
 
 typedef struct ts_storage_t ts_storage_t;
@@ -308,7 +310,6 @@ es_out_t *input_EsOutTimeshiftNew( input_thread_t *p_input, es_out_t *p_next_out
     p_out->pf_control = Control;
     p_out->pf_destroy = Destroy;
     p_out->p_sys      = p_sys;
-    p_out->b_sout     = p_input->p->p_sout != NULL;
 
     /* */
     p_sys->b_input_paused = false;
@@ -331,7 +332,7 @@ es_out_t *input_EsOutTimeshiftNew( input_thread_t *p_input, es_out_t *p_next_out
         p_sys->i_tmp_size_max = 50*1024*1024;
     else
         p_sys->i_tmp_size_max = __MAX( i_tmp_size_max, 1*1024*1024 );
-    msg_Dbg( p_input, "using timeshift granularity of %d MBytes",
+    msg_Dbg( p_input, "using timeshift granularity of %d MiB",
              (int)p_sys->i_tmp_size_max/(1024*1024) );
 
     char *psz_tmp_path = var_CreateGetNonEmptyString( p_input, "input-timeshift-path" );
@@ -507,7 +508,7 @@ static int ControlLockedSetPauseState( es_out_t *p_out, bool b_source_paused, bo
             /* XXX we may do it BUT it would be better to finish the clock clean up+improvments
              * and so be able to advertize correctly pace control property in access
              * module */
-            msg_Err( p_sys->p_input, "EsOutTimeshift does not work with streams that have space control" );
+            msg_Err( p_sys->p_input, "EsOutTimeshift does not work with streams that have pace control" );
         }
     }
 
@@ -542,7 +543,7 @@ static int ControlLockedSetRate( es_out_t *p_out, int i_src_rate, int i_rate )
             /* XXX we may do it BUT it would be better to finish the clock clean up+improvments
              * and so be able to advertize correctly pace control property in access
              * module */
-            msg_Err( p_sys->p_input, "EsOutTimeshift does not work with streams that have space control" );
+            msg_Err( p_sys->p_input, "EsOutTimeshift does not work with streams that have pace control" );
         }
 
     }
@@ -582,13 +583,13 @@ static int ControlLocked( es_out_t *p_out, int i_query, va_list args )
     case ES_OUT_SET_ES_BY_ID:
     case ES_OUT_RESTART_ES_BY_ID:
     case ES_OUT_SET_ES_DEFAULT_BY_ID:
+    case ES_OUT_GET_ES_OBJECTS_BY_ID:
     case ES_OUT_SET_DELAY:
     case ES_OUT_SET_RECORD_STATE:
         assert(0);
         return VLC_EGENERIC;
 
     /* Pass-through control */
-    case ES_OUT_SET_ACTIVE:
     case ES_OUT_SET_MODE:
     case ES_OUT_SET_GROUP:
     case ES_OUT_SET_PCR:
@@ -674,7 +675,25 @@ static int ControlLocked( es_out_t *p_out, int i_query, va_list args )
     {
         return ControlLockedSetFrameNext( p_out );
     }
+    case ES_OUT_GET_PCR_SYSTEM:
+    {
+        if( p_sys->b_delayed )
+            return VLC_EGENERIC;
 
+        mtime_t *pi_system = (mtime_t*)va_arg( args, mtime_t * );
+        mtime_t *pi_delay  = (mtime_t*)va_arg( args, mtime_t * );
+        return es_out_ControlGetPcrSystem( p_sys->p_out, pi_system, pi_delay );
+    }
+    case ES_OUT_MODIFY_PCR_SYSTEM:
+    {
+        const bool    b_absolute = va_arg( args, int );
+        const mtime_t i_system   = va_arg( args, mtime_t );
+
+        if( b_absolute && p_sys->b_delayed )
+            return VLC_EGENERIC;
+
+        return es_out_ControlModifyPcrSystem( p_sys->p_out, b_absolute, i_system );
+    }
     default:
         msg_Err( p_sys->p_input, "Unknown es_out_Control query !" );
         assert(0);
@@ -1049,14 +1068,14 @@ static ts_storage_t *TsStorageNew( const char *psz_tmp_path, int64_t i_tmp_size_
     p_storage->i_file_size = 0;
     p_storage->p_filew = GetTmpFile( &p_storage->psz_file, psz_tmp_path );
     if( p_storage->psz_file )
-        p_storage->p_filer = utf8_fopen( p_storage->psz_file, "rb" );
+        p_storage->p_filer = vlc_fopen( p_storage->psz_file, "rb" );
 
     /* */
     p_storage->i_cmd_w = 0;
     p_storage->i_cmd_r = 0;
     p_storage->i_cmd_max = 30000;
     p_storage->p_cmd = malloc( p_storage->i_cmd_max * sizeof(*p_storage->p_cmd) );
-    //fprintf( stderr, "\nSTORAGE name=%s size=%d kbytes\n", p_storage->psz_file, p_storage->i_cmd_max * sizeof(*p_storage->p_cmd) /1024 );
+    //fprintf( stderr, "\nSTORAGE name=%s size=%d KiB\n", p_storage->psz_file, p_storage->i_cmd_max * sizeof(*p_storage->p_cmd) /1024 );
 
     if( !p_storage->p_cmd || !p_storage->p_filew || !p_storage->p_filer )
     {
@@ -1084,7 +1103,7 @@ static void TsStorageDelete( ts_storage_t *p_storage )
 
     if( p_storage->psz_file )
     {
-        utf8_unlink( p_storage->psz_file );
+        vlc_unlink( p_storage->psz_file );
         free( p_storage->psz_file );
     }
 
@@ -1106,7 +1125,7 @@ static bool TsStorageIsFull( ts_storage_t *p_storage, const ts_cmd_t *p_cmd )
 {
     if( p_cmd && p_cmd->i_type == C_SEND && p_storage->i_cmd_w > 0 )
     {
-        size_t i_size = sizeof(*p_cmd->send.p_block) + p_cmd->send.p_block->i_buffer;
+        size_t i_size = sizeof(*p_cmd->u.send.p_block) + p_cmd->u.send.p_block->i_buffer;
 
         if( p_storage->i_file_size + i_size >= p_storage->i_file_max )
             return true;
@@ -1125,10 +1144,10 @@ static void TsStoragePushCmd( ts_storage_t *p_storage, const ts_cmd_t *p_cmd, bo
 
     if( cmd.i_type == C_SEND )
     {
-        block_t *p_block = cmd.send.p_block;
+        block_t *p_block = cmd.u.send.p_block;
 
-        cmd.send.p_block = NULL;
-        cmd.send.i_offset = ftell( p_storage->p_filew );
+        cmd.u.send.p_block = NULL;
+        cmd.u.send.i_offset = ftell( p_storage->p_filew );
 
         if( fwrite( p_block, sizeof(*p_block), 1, p_storage->p_filew ) != 1 )
         {
@@ -1162,7 +1181,7 @@ static void TsStoragePopCmd( ts_storage_t *p_storage, ts_cmd_t *p_cmd, bool b_fl
         block_t block;
 
         if( !b_flush &&
-            !fseek( p_storage->p_filer, p_cmd->send.i_offset, SEEK_SET ) &&
+            !fseek( p_storage->p_filer, p_cmd->u.send.i_offset, SEEK_SET ) &&
             fread( &block, sizeof(block), 1, p_storage->p_filer ) == 1 )
         {
             block_t *p_block = block_Alloc( block.i_buffer );
@@ -1173,15 +1192,15 @@ static void TsStoragePopCmd( ts_storage_t *p_storage, ts_cmd_t *p_cmd, bool b_fl
                 p_block->i_flags    = block.i_flags;
                 p_block->i_length   = block.i_length;
                 p_block->i_rate     = block.i_rate;
-                p_block->i_samples  = block.i_samples;
+                p_block->i_nb_samples = block.i_nb_samples;
                 p_block->i_buffer = fread( p_block->p_buffer, 1, block.i_buffer, p_storage->p_filer );
             }
-            p_cmd->send.p_block = p_block;
+            p_cmd->u.send.p_block = p_block;
         }
         else
         {
             //fprintf( stderr, "TsStoragePopCmd: %m\n" );
-            p_cmd->send.p_block = block_Alloc( 1 );
+            p_cmd->u.send.p_block = block_Alloc( 1 );
         }
     }
 }
@@ -1214,154 +1233,152 @@ static int CmdInitAdd( ts_cmd_t *p_cmd, es_out_id_t *p_es, const es_format_t *p_
 {
     p_cmd->i_type = C_ADD;
     p_cmd->i_date = mdate();
-    p_cmd->add.p_es = p_es;
+    p_cmd->u.add.p_es = p_es;
     if( b_copy )
     {
-        p_cmd->add.p_fmt = malloc( sizeof(*p_fmt) );
-        if( !p_cmd->add.p_fmt )
+        p_cmd->u.add.p_fmt = malloc( sizeof(*p_fmt) );
+        if( !p_cmd->u.add.p_fmt )
             return VLC_EGENERIC;
-        es_format_Copy( p_cmd->add.p_fmt, p_fmt );
+        es_format_Copy( p_cmd->u.add.p_fmt, p_fmt );
     }
     else
     {
-        p_cmd->add.p_fmt = (es_format_t*)p_fmt;
+        p_cmd->u.add.p_fmt = (es_format_t*)p_fmt;
     }
     return VLC_SUCCESS;
 }
 static void CmdExecuteAdd( es_out_t *p_out, ts_cmd_t *p_cmd )
 {
-    p_cmd->add.p_es->p_es = es_out_Add( p_out, p_cmd->add.p_fmt );
+    p_cmd->u.add.p_es->p_es = es_out_Add( p_out, p_cmd->u.add.p_fmt );
 }
 static void CmdCleanAdd( ts_cmd_t *p_cmd )
 {
-    es_format_Clean( p_cmd->add.p_fmt );
-    free( p_cmd->add.p_fmt );
+    es_format_Clean( p_cmd->u.add.p_fmt );
+    free( p_cmd->u.add.p_fmt );
 }
 
 static void CmdInitSend( ts_cmd_t *p_cmd, es_out_id_t *p_es, block_t *p_block )
 {
     p_cmd->i_type = C_SEND;
     p_cmd->i_date = mdate();
-    p_cmd->send.p_es = p_es;
-    p_cmd->send.p_block = p_block;
+    p_cmd->u.send.p_es = p_es;
+    p_cmd->u.send.p_block = p_block;
 }
 static int CmdExecuteSend( es_out_t *p_out, ts_cmd_t *p_cmd )
 {
-    block_t *p_block = p_cmd->send.p_block;
+    block_t *p_block = p_cmd->u.send.p_block;
 
-    p_cmd->send.p_block = NULL;
+    p_cmd->u.send.p_block = NULL;
 
     if( p_block )
     {
-        if( p_cmd->send.p_es->p_es )
-            return es_out_Send( p_out, p_cmd->send.p_es->p_es, p_block );
+        if( p_cmd->u.send.p_es->p_es )
+            return es_out_Send( p_out, p_cmd->u.send.p_es->p_es, p_block );
         block_Release( p_block );
     }
     return VLC_EGENERIC;
 }
 static void CmdCleanSend( ts_cmd_t *p_cmd )
 {
-    if( p_cmd->send.p_block )
-        block_Release( p_cmd->send.p_block );
+    if( p_cmd->u.send.p_block )
+        block_Release( p_cmd->u.send.p_block );
 }
 
 static int CmdInitDel( ts_cmd_t *p_cmd, es_out_id_t *p_es )
 {
     p_cmd->i_type = C_DEL;
     p_cmd->i_date = mdate();
-    p_cmd->del.p_es = p_es;
+    p_cmd->u.del.p_es = p_es;
     return VLC_SUCCESS;
 }
 static void CmdExecuteDel( es_out_t *p_out, ts_cmd_t *p_cmd )
 {
-    if( p_cmd->del.p_es->p_es )
-        es_out_Del( p_out, p_cmd->del.p_es->p_es );
-    free( p_cmd->del.p_es );
+    if( p_cmd->u.del.p_es->p_es )
+        es_out_Del( p_out, p_cmd->u.del.p_es->p_es );
+    free( p_cmd->u.del.p_es );
 }
 
 static int CmdInitControl( ts_cmd_t *p_cmd, int i_query, va_list args, bool b_copy )
 {
     p_cmd->i_type = C_CONTROL;
     p_cmd->i_date = mdate();
-    p_cmd->control.i_query = i_query;
+    p_cmd->u.control.i_query = i_query;
 
     switch( i_query )
     {
     /* Pass-through control */
-    case ES_OUT_SET_ACTIVE:  /* arg1= bool                     */
-        p_cmd->control.b_bool = (bool)va_arg( args, int );
-        break;
-
     case ES_OUT_SET_MODE:    /* arg1= int                            */
     case ES_OUT_SET_GROUP:   /* arg1= int                            */
     case ES_OUT_DEL_GROUP:   /* arg1=int i_group */
-        p_cmd->control.i_int = (int)va_arg( args, int );
+        p_cmd->u.control.u.i_int = (int)va_arg( args, int );
         break;
 
     case ES_OUT_SET_PCR:                /* arg1=int64_t i_pcr(microsecond!) (using default group 0)*/
     case ES_OUT_SET_NEXT_DISPLAY_TIME:  /* arg1=int64_t i_pts(microsecond) */
-        p_cmd->control.i_i64 = (int64_t)va_arg( args, int64_t );
+        p_cmd->u.control.u.i_i64 = (int64_t)va_arg( args, int64_t );
         break;
 
     case ES_OUT_SET_GROUP_PCR:          /* arg1= int i_group, arg2=int64_t i_pcr(microsecond!)*/
-        p_cmd->control.int_i64.i_int = (int)va_arg( args, int );
-        p_cmd->control.int_i64.i_i64 = (int64_t)va_arg( args, int64_t );
+        p_cmd->u.control.u.int_i64.i_int = (int)va_arg( args, int );
+        p_cmd->u.control.u.int_i64.i_i64 = (int64_t)va_arg( args, int64_t );
         break;
 
     case ES_OUT_SET_ES_SCRAMBLED_STATE:
-        p_cmd->control.es_bool.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
-        p_cmd->control.es_bool.b_bool = (bool)va_arg( args, int );
+        p_cmd->u.control.u.es_bool.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
+        p_cmd->u.control.u.es_bool.b_bool = (bool)va_arg( args, int );
         break;
 
     case ES_OUT_RESET_PCR:           /* no arg */
         break;
 
     case ES_OUT_SET_META:        /* arg1=const vlc_meta_t* */
-    case ES_OUT_SET_GROUP_META:  /* arg1=int i_group arg2=vlc_meta_t* */
+    case ES_OUT_SET_GROUP_META:  /* arg1=int i_group arg2=const vlc_meta_t* */
     {
         if( i_query == ES_OUT_SET_GROUP_META )
-            p_cmd->control.int_meta.i_int = (int)va_arg( args, int );
-        vlc_meta_t *p_meta = (vlc_meta_t*)va_arg( args, vlc_meta_t * );
+            p_cmd->u.control.u.int_meta.i_int = (int)va_arg( args, int );
+        const vlc_meta_t *p_meta = va_arg( args, const vlc_meta_t * );
 
         if( b_copy )
         {
-            p_cmd->control.int_meta.p_meta = vlc_meta_New();
-            if( !p_cmd->control.int_meta.p_meta )
+            p_cmd->u.control.u.int_meta.p_meta = vlc_meta_New();
+            if( !p_cmd->u.control.u.int_meta.p_meta )
                 return VLC_EGENERIC;
-            vlc_meta_Merge( p_cmd->control.int_meta.p_meta, p_meta );
+            vlc_meta_Merge( p_cmd->u.control.u.int_meta.p_meta, p_meta );
         }
         else
         {
-            p_cmd->control.int_meta.p_meta = p_meta;
+            /* The cast is only needed to avoid warning */
+            p_cmd->u.control.u.int_meta.p_meta = (vlc_meta_t*)p_meta;
         }
         break;
     }
 
-    case ES_OUT_SET_GROUP_EPG:   /* arg1=int i_group arg2=vlc_epg_t* */
+    case ES_OUT_SET_GROUP_EPG:   /* arg1=int i_group arg2=const vlc_epg_t* */
     {
-        p_cmd->control.int_epg.i_int = (int)va_arg( args, int );
-        vlc_epg_t *p_epg = (vlc_epg_t*)va_arg( args, vlc_epg_t * );
+        p_cmd->u.control.u.int_epg.i_int = (int)va_arg( args, int );
+        const vlc_epg_t *p_epg = va_arg( args, const vlc_epg_t * );
 
         if( b_copy )
         {
-            p_cmd->control.int_epg.p_epg = vlc_epg_New( p_epg->psz_name );
-            if( !p_cmd->control.int_epg.p_epg )
+            p_cmd->u.control.u.int_epg.p_epg = vlc_epg_New( p_epg->psz_name );
+            if( !p_cmd->u.control.u.int_epg.p_epg )
                 return VLC_EGENERIC;
             for( int i = 0; i < p_epg->i_event; i++ )
             {
                 vlc_epg_event_t *p_evt = p_epg->pp_event[i];
 
-                vlc_epg_AddEvent( p_cmd->control.int_epg.p_epg,
+                vlc_epg_AddEvent( p_cmd->u.control.u.int_epg.p_epg,
                                   p_evt->i_start, p_evt->i_duration,
                                   p_evt->psz_name,
                                   p_evt->psz_short_description, p_evt->psz_description );
             }
-            vlc_epg_SetCurrent( p_cmd->control.int_epg.p_epg,
+            vlc_epg_SetCurrent( p_cmd->u.control.u.int_epg.p_epg,
                                 p_epg->p_current ? p_epg->p_current->i_start : -1 );
         }
         else
         {
-            p_cmd->control.int_epg.p_epg = p_epg;
+            /* The cast is only needed to avoid warning */
+            p_cmd->u.control.u.int_epg.p_epg = (vlc_epg_t*)p_epg;
         }
         break;
     }
@@ -1370,29 +1387,29 @@ static int CmdInitControl( ts_cmd_t *p_cmd, int i_query, va_list args, bool b_co
     case ES_OUT_SET_ES:      /* arg1= es_out_id_t*                   */
     case ES_OUT_RESTART_ES:  /* arg1= es_out_id_t*                   */
     case ES_OUT_SET_ES_DEFAULT: /* arg1= es_out_id_t*                */
-        p_cmd->control.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
+        p_cmd->u.control.u.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
         break;
 
     case ES_OUT_SET_ES_STATE:/* arg1= es_out_id_t* arg2=bool   */
-        p_cmd->control.es_bool.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
-        p_cmd->control.es_bool.b_bool = (bool)va_arg( args, int );
+        p_cmd->u.control.u.es_bool.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
+        p_cmd->u.control.u.es_bool.b_bool = (bool)va_arg( args, int );
         break;
 
     case ES_OUT_SET_ES_FMT:     /* arg1= es_out_id_t* arg2=es_format_t* */
     {
-        p_cmd->control.es_fmt.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
+        p_cmd->u.control.u.es_fmt.p_es = (es_out_id_t*)va_arg( args, es_out_id_t * );
         es_format_t *p_fmt = (es_format_t*)va_arg( args, es_format_t * );
 
         if( b_copy )
         {
-            p_cmd->control.es_fmt.p_fmt = malloc( sizeof(*p_fmt) );
-            if( !p_cmd->control.es_fmt.p_fmt )
+            p_cmd->u.control.u.es_fmt.p_fmt = malloc( sizeof(*p_fmt) );
+            if( !p_cmd->u.control.u.es_fmt.p_fmt )
                 return VLC_EGENERIC;
-            es_format_Copy( p_cmd->control.es_fmt.p_fmt, p_fmt );
+            es_format_Copy( p_cmd->u.control.u.es_fmt.p_fmt, p_fmt );
         }
         else
         {
-            p_cmd->control.es_fmt.p_fmt = p_fmt;
+            p_cmd->u.control.u.es_fmt.p_fmt = p_fmt;
         }
         break;
     }
@@ -1402,18 +1419,20 @@ static int CmdInitControl( ts_cmd_t *p_cmd, int i_query, va_list args, bool b_co
         mtime_t i_time = (mtime_t)va_arg( args, mtime_t );
         mtime_t i_length = (mtime_t)va_arg( args, mtime_t );
 
-        p_cmd->control.times.f_position = f_position;
-        p_cmd->control.times.i_time = i_time;
-        p_cmd->control.times.i_length = i_length;
+        p_cmd->u.control.u.times.f_position = f_position;
+        p_cmd->u.control.u.times.i_time = i_time;
+        p_cmd->u.control.u.times.i_length = i_length;
         break;
     }
     case ES_OUT_SET_JITTER:
     {
         mtime_t i_pts_delay = (mtime_t)va_arg( args, mtime_t );
+        mtime_t i_pts_jitter = (mtime_t)va_arg( args, mtime_t );
         int     i_cr_average = (int)va_arg( args, int );
 
-        p_cmd->control.jitter.i_pts_delay = i_pts_delay;
-        p_cmd->control.jitter.i_cr_average = i_cr_average;
+        p_cmd->u.control.u.jitter.i_pts_delay = i_pts_delay;
+        p_cmd->u.control.u.jitter.i_pts_jitter = i_pts_jitter;
+        p_cmd->u.control.u.jitter.i_cr_average = i_cr_average;
         break;
     }
 
@@ -1426,66 +1445,64 @@ static int CmdInitControl( ts_cmd_t *p_cmd, int i_query, va_list args, bool b_co
 }
 static int CmdExecuteControl( es_out_t *p_out, ts_cmd_t *p_cmd )
 {
-    const int i_query = p_cmd->control.i_query;
+    const int i_query = p_cmd->u.control.i_query;
 
     switch( i_query )
     {
     /* Pass-through control */
-    case ES_OUT_SET_ACTIVE:  /* arg1= bool                     */
-        return es_out_Control( p_out, i_query, p_cmd->control.b_bool );
-
     case ES_OUT_SET_MODE:    /* arg1= int                            */
     case ES_OUT_SET_GROUP:   /* arg1= int                            */
     case ES_OUT_DEL_GROUP:   /* arg1=int i_group */
-        return es_out_Control( p_out, i_query, p_cmd->control.i_int );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.i_int );
 
     case ES_OUT_SET_PCR:                /* arg1=int64_t i_pcr(microsecond!) (using default group 0)*/
     case ES_OUT_SET_NEXT_DISPLAY_TIME:  /* arg1=int64_t i_pts(microsecond) */
-        return es_out_Control( p_out, i_query, p_cmd->control.i_i64 );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.i_i64 );
 
     case ES_OUT_SET_GROUP_PCR:          /* arg1= int i_group, arg2=int64_t i_pcr(microsecond!)*/
-        return es_out_Control( p_out, i_query, p_cmd->control.int_i64.i_int,
-                                               p_cmd->control.int_i64.i_i64 );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.int_i64.i_int,
+                                               p_cmd->u.control.u.int_i64.i_i64 );
 
     case ES_OUT_RESET_PCR:           /* no arg */
         return es_out_Control( p_out, i_query );
 
-    case ES_OUT_SET_GROUP_META:  /* arg1=int i_group arg2=vlc_meta_t* */
-        return es_out_Control( p_out, i_query, p_cmd->control.int_meta.i_int,
-                                               p_cmd->control.int_meta.p_meta );
+    case ES_OUT_SET_GROUP_META:  /* arg1=int i_group arg2=const vlc_meta_t* */
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.int_meta.i_int,
+                                               p_cmd->u.control.u.int_meta.p_meta );
 
-    case ES_OUT_SET_GROUP_EPG:   /* arg1=int i_group arg2=vlc_epg_t* */
-        return es_out_Control( p_out, i_query, p_cmd->control.int_epg.i_int,
-                                               p_cmd->control.int_epg.p_epg );
+    case ES_OUT_SET_GROUP_EPG:   /* arg1=int i_group arg2=const vlc_epg_t* */
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.int_epg.i_int,
+                                               p_cmd->u.control.u.int_epg.p_epg );
 
     case ES_OUT_SET_ES_SCRAMBLED_STATE: /* arg1=int es_out_id_t* arg2=bool */
-        return es_out_Control( p_out, i_query, p_cmd->control.es_bool.p_es->p_es,
-                                               p_cmd->control.es_bool.b_bool );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.es_bool.p_es->p_es,
+                                               p_cmd->u.control.u.es_bool.b_bool );
 
     case ES_OUT_SET_META:  /* arg1=const vlc_meta_t* */
-        return es_out_Control( p_out, i_query, p_cmd->control.int_meta.p_meta );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.int_meta.p_meta );
 
     /* Modified control */
     case ES_OUT_SET_ES:      /* arg1= es_out_id_t*                   */
     case ES_OUT_RESTART_ES:  /* arg1= es_out_id_t*                   */
     case ES_OUT_SET_ES_DEFAULT: /* arg1= es_out_id_t*                */
-        return es_out_Control( p_out, i_query, p_cmd->control.p_es->p_es );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.p_es->p_es );
 
     case ES_OUT_SET_ES_STATE:/* arg1= es_out_id_t* arg2=bool   */
-        return es_out_Control( p_out, i_query, p_cmd->control.es_bool.p_es->p_es,
-                                               p_cmd->control.es_bool.b_bool );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.es_bool.p_es->p_es,
+                                               p_cmd->u.control.u.es_bool.b_bool );
 
     case ES_OUT_SET_ES_FMT:     /* arg1= es_out_id_t* arg2=es_format_t* */
-        return es_out_Control( p_out, i_query, p_cmd->control.es_fmt.p_es->p_es,
-                                               p_cmd->control.es_fmt.p_fmt );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.es_fmt.p_es->p_es,
+                                               p_cmd->u.control.u.es_fmt.p_fmt );
 
     case ES_OUT_SET_TIMES:
-        return es_out_Control( p_out, i_query, p_cmd->control.times.f_position,
-                                               p_cmd->control.times.i_time,
-                                               p_cmd->control.times.i_length );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.times.f_position,
+                                               p_cmd->u.control.u.times.i_time,
+                                               p_cmd->u.control.u.times.i_length );
     case ES_OUT_SET_JITTER:
-        return es_out_Control( p_out, i_query, p_cmd->control.jitter.i_pts_delay,
-                                               p_cmd->control.jitter.i_cr_average );
+        return es_out_Control( p_out, i_query, p_cmd->u.control.u.jitter.i_pts_delay,
+                                               p_cmd->u.control.u.jitter.i_pts_jitter,
+                                               p_cmd->u.control.u.jitter.i_cr_average );
 
     default:
         assert(0);
@@ -1494,22 +1511,22 @@ static int CmdExecuteControl( es_out_t *p_out, ts_cmd_t *p_cmd )
 }
 static void CmdCleanControl( ts_cmd_t *p_cmd )
 {
-    if( ( p_cmd->control.i_query == ES_OUT_SET_GROUP_META ||
-          p_cmd->control.i_query == ES_OUT_SET_META ) &&
-        p_cmd->control.int_meta.p_meta )
+    if( ( p_cmd->u.control.i_query == ES_OUT_SET_GROUP_META ||
+          p_cmd->u.control.i_query == ES_OUT_SET_META ) &&
+        p_cmd->u.control.u.int_meta.p_meta )
     {
-        vlc_meta_Delete( p_cmd->control.int_meta.p_meta );
+        vlc_meta_Delete( p_cmd->u.control.u.int_meta.p_meta );
     }
-    else if( p_cmd->control.i_query == ES_OUT_SET_GROUP_EPG &&
-             p_cmd->control.int_epg.p_epg )
+    else if( p_cmd->u.control.i_query == ES_OUT_SET_GROUP_EPG &&
+             p_cmd->u.control.u.int_epg.p_epg )
     {
-        vlc_epg_Delete( p_cmd->control.int_epg.p_epg );
+        vlc_epg_Delete( p_cmd->u.control.u.int_epg.p_epg );
     }
-    else if( p_cmd->control.i_query == ES_OUT_SET_ES_FMT &&
-             p_cmd->control.es_fmt.p_fmt )
+    else if( p_cmd->u.control.i_query == ES_OUT_SET_ES_FMT &&
+             p_cmd->u.control.u.es_fmt.p_fmt )
     {
-        es_format_Clean( p_cmd->control.es_fmt.p_fmt );
-        free( p_cmd->control.es_fmt.p_fmt );
+        es_format_Clean( p_cmd->u.control.u.es_fmt.p_fmt );
+        free( p_cmd->u.control.u.es_fmt.p_fmt );
     }
 }
 
@@ -1523,9 +1540,9 @@ static char *GetTmpPath( char *psz_path )
     {
         /* Make sure that the path exists and is a directory */
         struct stat s;
-        const int i_ret = utf8_stat( psz_path, &s );
+        const int i_ret = vlc_stat( psz_path, &s );
 
-        if( i_ret < 0 && !utf8_mkdir( psz_path, 0600 ) )
+        if( i_ret < 0 && !vlc_mkdir( psz_path, 0600 ) )
             return psz_path;
         else if( i_ret == 0 && ( s.st_mode & S_IFDIR ) )
             return psz_path;
@@ -1580,7 +1597,7 @@ static FILE *GetTmpFile( char **ppsz_file, const char *psz_path )
         return NULL;
 
     /* */
-    fd = utf8_mkstemp( psz_name );
+    fd = vlc_mkstemp( psz_name );
     *ppsz_file = psz_name;
 
     if( fd < 0 )
